from Tools.SecurityModuleCore import SecurityToken
from Models import WorkflowEntry, EntryStatus, Location

class ModifiableRequest (object):
    """
    This functor can be used to decide wether or not a requet can be modified in the current 
    context or not.
    """
    def __init__ (self):
        pass
    def __call__ (self, context, entry, **args):
        securityApi = context.SecurityDBApi ()
        token = SecurityToken ()
        # If the request is approved, it cannot be changed.
        # TODO: change it so that it checks wether or not the prodmanager has seen it.
        if entry.status == EntryStatus.APPROVED or entry.status == EntryStatus.APPROVED_PRODMANAGER:
            print "Object in APPROVED status. Not changing"
            return False
        # First of all check if the user is a Production Manager
        if securityApi.hasGroupResponsibility (token.dn, "ProdRequest", "Production Manager"):
            return True
        # Then check if the request is owned by 
        token = SecurityToken ()
        if entry.workflow.owner_id == token.userId:
            return True
        print "User not owner of the object. Nothing changed."
        return False

class OperatorsRequest (object):
    """
    This functor can be used to decide if an Production Operator owns the request.
    Production Manager is ok as well.
    """
    def __init__ (self):
        pass

    def __call__ (self, context, entry, **args):
        securityApi = context.SecurityDBApi ()
        token = SecurityToken ()
        # If it is a production manager. Allow him.
        if securityApi.hasGroupResponsibility (token.dn, "ProdRequest", "Production Manager"):
            return True
        # Check if the user is a Production Operator in any of the 
        # regions suggested for the entry. 
        userGroups = securityApi.groupsForRole (token.dn, "Production Operator")
        if not len (userGroups):
            return False
        for location in entry.location_hints:
            for userGroup in userGroups.values ():
                availableLocation = Location.select (Location.q.user_group == userGroup["id"])
                if availableLocation.count ():
                    if location.id == availableLocation[0].id:
                        return True
        return False
