from Models.Schema import *

from Tools.Functors import AlwaysFalse

from Framework import rpccall, xmlBooleanResponse, templatepage, templateXML, exposeSerialized
from Framework import Controller
from Framework.Controller import require_args
from Framework.Utilities import selectFromDict
from Framework.Application import Application as FrameworkApplication
from cherrypy import expose
import cherrypy
from Tools.SecurityModuleCore import is_authorized, is_authenticated, has_role
from Tools.SecurityModuleCore import Role, Group, SecurityToken
from Tools.SecurityModuleCore import check_ownership

from Tools.Functors import FetchFromArgs

from Modules.Utilities.Serializers.XML import PythonListSerializer, SQLObjectSerializer

from Framework.PluginManager import DeclarePlugin
from ProdRequestCfg import ProdRequestCfg
from RequestPolicies import OperatorsRequest, ModifiableRequest
from Tools import catch_exception

from os.path import join, dirname
import random
import datetime
from cherrypy import HTTPRedirect
from ProdRequestCommon.DataBase.ProdRequestApi import ProdRequestApi
from ProdRequestCommon.DataBase.ProdRequestApi import ProdRequestGUIApi
from ProdRequestCommon.DataBase.ProdRequestApi import buildQueryFromUrl
import re
import os

#TODO: expire session data after a defined period of time.

def lookupInState (state, name):
  for substate in state:
    if substate["label"] == name: return substate["state"]

def json (func):
  def wrapper (self, *args, **kwds):
    funcName = func.__name__
    data = func (self, *args, **kwds)
    cherrypy.response.headers['Content-Length'] = str (len (data))
    cherrypy.response.headers['Content-Type'] = "application/json"
    return str (data)
  wrapper.__doc__ = func.__doc__
  wrapper.__name__ = func.__name__
  wrapper.exposed = True
  return wrapper

class StateBuilder (object):
  def __init__(self, apiPR, apiGUI, sessionId, partialState):
    self.apiPR = apiPR
    self.apiGUI = apiGUI
    self.__sessionId = sessionId
    self.__state = []
    self.__partialState = partialState
    
  def generateState(self, pluginName):
    if self.__partialState and (pluginName in self.__partialState):
      return self.nonUpdatingState(pluginName)
    try:
      updater = getattr(self, "stateFor" +  pluginName.encode ('ascii'))
    except AttributeError:
      return {}
    return updater()

  def nonUpdatingState(self, pluginName):
    return {pluginName: {"notUpdating": true}}

  def stateForMastHead(self):
    return {"serviceName": "prodrequest", 
            "user": SecurityToken().dn,
            "serviceColor": "#FFFFFF",
            "serviceBackgroundColor": "#FF370A",
            "background": "#FFE9B3",
            "color": "#000000"}

  def stateForAvailableColumns(self):
    columns = self.apiGUI.getDataObject (['label', "fieldName", "type"],
            """SELECT cd.label, cd.field, ct.label 
               FROM column_description cd
               INNER JOIN column_type ct ON ct.id = cd.type_id;""", {})
    return columns
    
  def stateForMainTable(self):
    # TODO: make the default view user specific???
    GET_TABLE_INFO = """SELECT mts.page, mts.pageSize, mts.query
       FROM main_table_state mts
       WHERE mts.session_id = :session
       LIMIT 1;"""
    result = self.apiGUI.getDataObject (["page", "pageSize", "query"], 
                                      GET_TABLE_INFO, 
                                      {"session": self.__sessionId})[0]
    return result

  def stateForMainTableContents(self):
    mainTableState = lookupInState(self.__state, 'MainTable')
    queryParts = mainTableState["query"].split ("?")
    fields = queryParts[0]
    conditions = (len (queryParts) > 1) and queryParts[1] or ""
    limit = mainTableState["pageSize"]
    offset = mainTableState["pageSize"] * mainTableState["page"] 
    queryUrl = fields + "?l=%s&off=%s&%s" % (limit, offset, conditions)
    countUrl = fields + "?op=count(%%s)&l=-1&off=0&%s" % (conditions)
    entries = self.apiPR.queryByUrl(queryUrl.strip ("&"))
    mainTableState["size"] = self.apiPR.queryByUrl(countUrl.strip ("&"))[0]
    return entries    
  
  def stateForQueryBuilder(self):
    GET_TEXT_QUERY_INFO = """SELECT mts.textQuery, mts.textQueryColumn
                             FROM main_table_state mts
                             WHERE mts.session_id = :session
                             LIMIT 1;"""
    return self.apiGUI.getDataObject (["textQuery", "textQueryColumn"],
                                      GET_TEXT_QUERY_INFO,
                                      {"session": self.__sessionId})[0]
  
  def stateForRequestStatusUpdater (self):
    mainTableState = lookupInState(self.__state, 'MainTable')
    queryParts = mainTableState["query"].split ("?")
    fields = queryParts[0]
    conditions = (len (queryParts) > 1) and queryParts[1] or ""
    limit = mainTableState["pageSize"]
    offset = mainTableState["pageSize"] * mainTableState["page"] 
    statusPath = "current_log_entry[history_log].event_type.label"
    queryUrl = "%s?l=%s&off=%s&%s" % (join (fields, statusPath), 
                                      limit, offset, conditions)
    fields, query, binds = buildQueryFromUrl(queryUrl.strip("&"))
    entries = self.apiPR.getDataObject(fields,query, binds)
    return [e[u'event_type.label'] for e in entries]
    
  def stateForRequestEditor (self):
    GET_CURRENT_REQUEST_ID = """SELECT mts.selected_request_id
    FROM main_table_state mts
    WHERE mts.session_id = :session
    """
    
    GET_HISTORY_LOG = """SELECT et.label, hl.time, ri.owner_id FROM
    history_log hl
    INNER JOIN event_type et ON et.id = hl.event_type_id
    INNER JOIN request r ON r.id = hl.request_id
    INNER JOIN requestor_info ri ON ri.id = r.requestor_info_id
    WHERE hl.request_id = :request_id"""
    
    requestIds = self.apiGUI.getDataObject("id", 
                                           GET_CURRENT_REQUEST_ID, 
                                           {"session": self.__sessionId})
    
    if not requestIds:
      return {}
    request_id = requestIds[0]["id"]
    
    if not request_id:
      return {}
      
    result = self.apiPR.getRequestInfo (request_id)

    result['history'] = [h 
                         for h in self.apiPR.getDataObject(["event_type", 
                                                            "time", 
                                                            "who"],
                                                            GET_HISTORY_LOG,
                                                            {"request_id": request_id})]
    result['selectedRequest'] = request_id
    return result
  
  def stateForErrorLog (self):
    GET_ERROR_LOG = """SELECT ed.description, sel.user, sel.time 
                       FROM session_error_log sel
                       INNER JOIN error_description ed ON ed.id = sel.error_description_id
                       WHERE sel.session_id = :session_id
                       ORDER BY sel.id DESC
                       LIMIT 1;"""
    result = self.apiGUI.getDataObject(["description", "user", "time"],
                                       GET_ERROR_LOG,
                                       {"session_id": self.__sessionId})
    if not result:
      return []
    return result[0]
    
  def recomputeState(self):
    self.__state = []
    plugins = self.apiGUI.getDataObject ("label", 
                                         """SELECT ep.name FROM enabled_plugin ep
                                            WHERE ep.session = :session 
                                            ORDER BY ep.ordering""", 
                                         {"session": self.__sessionId})
    for pluginInfo in plugins:
      label = pluginInfo["label"]
      self.__state.append({"label": label, "state": self.generateState(label)})
    return self.__state



class ReturnErrorMessage(object):
  def __init__ (self, errorName):
    self.__errorName = errorName

  def __call__ (self, *args, **kwds):
    sessionId, args = self.controller().parseArgs(args[1:])
    self.controller().apiGUI.setSessionError(sessionId, 
                                             SecurityToken().dn, 
                                             self.__errorName)
    return self.controller().state(sessionId, args)

class ProdRequest(Controller):
  def __init__(self, context):
    Controller.__init__(self, context, __file__)
    self.apiPR = ProdRequestApi()
    self.apiGUI = ProdRequestGUIApi()
    self.apiPR.connect("prodrequest_new.db", "sqlite")
    self.apiGUI.connect("prodrequest_gui.db", "sqlite")
    ReturnErrorMessage.controller = lambda x : self

  def parseArgs (self, args):
    if not len(args):
      raise HTTPRedirect(self.__getNewSessionUrl(SecurityToken().dn))
    nonEmpty = [a for a in args if a]
    print nonEmpty[0]
    return nonEmpty[0], nonEmpty[1:]
    
  def readyToRun (self):
    self.sessions = {}
    self.baseUrl = self.context.CmdLineArgs ().opts.baseUrl.strip ("/")
    self.securityApi = self.context.SecurityDBApi ()

  def __getClonedSessionUrl (self, owner, originalSession):
    sessionId = ''.join([random.choice('abcdefghijklmnopqrstuvwxyz') for x in xrange(10)])
    if self.apiGUI.hasSession (sessionId):
      return self.__getClonedSessionUrl(owner, originalSession)
    sessionUrl = self.baseUrl + join("/ProdRequest/session", sessionId)
    sessionTime = datetime.datetime.now()
    sessionOwner = owner
    self.apiGUI.cloneSession(originalSession, sessionId, owner)
    return sessionUrl
  
  def __getNewSessionUrl(self, owner):
    return self.__getClonedSessionUrl(owner, '*default*')
  
  @expose
  def index(self, *args, **kwds):
    """ Redirects to a new session
    """
    url = self.__getNewSessionUrl (SecurityToken ().dn)
    raise HTTPRedirect (url)
    
  @expose
  def session (self, *args, **kwds):
    sessionId, args = self.parseArgs(args)
    # TODO: duplicate session if the owner is not the same.
    if not self.apiGUI.checkSessionOwner(sessionId, SecurityToken().dn):
      raise HTTPRedirect(self.__getClonedSessionUrl(SecurityToken().dn, sessionId))
    f = file(join(dirname(__file__), "html/index.html"))
    return f.read()
  
  def state (self, sessionId, *partialState):
    """ Returns the complete or partial state associated to a a given 
        sessionId. 
        * In order to return the full state, simply do not put any 
        extra argument. 
        * In order to have a partial update, add the extra arguments for the 
          plugins to be updated normally, the others will have the form:
          {'PluginName': {unchanged: 1}}
    """
    builder = StateBuilder(self.apiPR, self.apiGUI, sessionId, partialState)
    return repr (builder.recomputeState()).replace ("u'","'")
  
  @json
  def autoUpdate (self, *args, **kwds):
    print self.parseArgs (args)
    return self.state (*self.parseArgs(args))
  
  @json
  def updateQueryConditions (self, *args, **kwds):
    """q: the query
       tq: the main text search
       tqc: the column on which perform the text search.
    """
    sessionId, args = self.parseArgs(args)
    self.apiGUI.updateQueryConditions (sessionId, "q=%s" % kwds["q"], 
                                       textQuery=kwds["tq"],
                                       textQueryColumn=kwds["tqc"])
    return self.state(sessionId, args)
  
  @json
  def updateTablePage(self, *args, **kwds):
    """ page: current page
    """
    sessionId, args = self.parseArgs(args)
    try:
      page = int (kwds["page"])
      self.apiGUI.updateTablePage (sessionId, page)
    except:
      pass
    return self.state (sessionId, args)
  
  @json
  def updateTextQueryField (self, *args, **kwds):
    """ field: new field on which perform the text search.
    """
    sessionId, args = self.parseArgs(args)
    self.apiGUI.updateTextQueryField (sessionId, kwds["field"])
    self.apiGUI.updateTablePage (sessionId, 0)
    return self.state(sessionId, args)
  
  @json
  def addColumn (self, *args, **kwds):
    """ columnLabel: label of the column to be added.
    """
    sessionId, args = self.parseArgs(args)
    self.apiGUI.addColumn(sessionId, kwds["columnLabel"])
    return self.state(sessionId, args)

  @json
  def removeColumn (self, *args, **kwds):
    """ columnLabel: label of the column to be added.
    """
    sessionId, args = self.parseArgs(args)
    self.apiGUI.removeColumn (sessionId, kwds["columnLabel"])
    return self.state(sessionId, args)
  
  @expose
  def selectRequest(self, *args, **kwds):
    sessionId, args = self.parseArgs(args)
    self.apiGUI.selectRequest(sessionId, kwds["request_id"])
    return self.state(sessionId, args)

  @json
  @is_authorized (Role ("Production Manager"), 
                  Group ("ProdRequest"), 
                  onFail=ReturnErrorMessage("CANNOT_DENY"))
  def denyRequest(self, *args, **kwds):
    sessionId, args = self.parseArgs(args)
    self.apiPR.denyRequest(kwds["id"])
    return self.state(sessionId, args)

  @json
  @is_authorized (Role ("Production Manager"), 
                  Group ("ProdRequest"), 
                  onFail=ReturnErrorMessage("CANNOT_APPROVE"))
  def approveRequest(self, *args, **kwds):
    sessionId, args = self.parseArgs(args)
    self.apiPR.approveRequest(kwds["id"],"PRODMANAGER")
    return self.state(sessionId, args)

  @json
  def clearSessionError(self, *args, **kwds):
    sessionId, args = self.parseArgs(args)
    self.apiGUI.setSessionError(sessionId, SecurityToken().dn, "NO_ERROR")
    return self.state(sessionId, args)

  @expose
  @is_authorized (Role ("Production Manager"), 
                  Group ("ProdRequest"), 
                  onFail=ReturnErrorMessage("CANNOT_DELETE"))
  def deleteRequest(self, *args, **kwds):
    sessionId, args = self.parseArgs(args)
    self.apiPR.deleteRequest(kwds["id"])
    return self.state(sessionId, args)

  @expose
  @is_authorized (Role ("Production Manager"), 
                  Group ("ProdRequest"), 
                  onFail=ReturnErrorMessage("CANNOT_ARCHIVE"))
  def archiveRequest(self, *args, **kwds):
    sessionId, args = self.parseArgs(args)
    self.apiPR.archiveRequest(kwds["id"])
    return self.state(sessionId, args)
  
  @expose
  def workflowSpecDict(self, *args):
    return repr (self.apiPR.getWorkflowSpecInfo(int(args[0])))

  @expose
  @is_authenticated(onFail=bool)
  def createRequest(self, *args, **kwds):
    securityApi = self.context.SecurityDBApi ()
    kwds["user_id"] = securityApi.getIDFromUsername(SecurityToken().dn)
    if not kwds["version"]:
      return False
    kwds["cmssw_version"] = kwds["version"]
    
    fields = ["priority_hint", "preferred_consumer", "architecture",
              "cmssw_version", "physics_group", "request_type", 
              "request_category", "request_type"]
    
    GET_DEFAULTS = """SELECT %s
                      FROM default_values d;""" % ", ".join(["d.%s" % f 
                                                            for f in fields])
                      
    defaults = self.apiGUI.getDataObject (fields, GET_DEFAULTS)[0]
    print "Defaults", defaults
    for key in defaults.iterkeys():
      if key not in kwds:
        kwds[key] = defaults[key]
    print "Actual values", kwds
    if kwds["request_type"] == "cmsGen":
      if generator_name not in kwds or generator_cfg not in kwds:
        return False
    
    if "requested_events" in kwds:
      kwds["requested_events"] = int(kwds["requested_events"])
    
    # TODO: Do all this on the server side.
    # TODO: Rename cmssw_to_prodcommon_rules to singular.
    if "prodcommon_version" not in kwds:
      GET_PRODCOMMON_REGEXP = """SELECT cpr.re, pc.label
                                 FROM cmssw_to_prodcommon_rules cpr
                                 INNER JOIN prodcommon pc ON pc.id = cpr.prodcommon_id"""
      for expr, prodcommon in self.apiPR.getDataTable (GET_PRODCOMMON_REGEXP):
        if re.match (expr, kwds["cmssw_version"]):
          kwds["prodcommon_version"] = prodcommon
          break
    self.apiPR.createRequest (**kwds)
    return True

  @templateXML
  @require_args ("lastSeen", onFail=list)
  def getRequestSpecsSlice (self, lastSeen, consumer="PRODMANAGER", maxRequests=10, **args):
    securityApi = self.securityApi
    GET_REQUEST_SPECS_SLICE = """SELECT pi.requested_events, ar.id, r.id, ri.owner_id, pi.priority_hint
FROM approved_request ar
INNER JOIN request r ON r.id = ar.request_id
INNER JOIN production_info pi ON pi.id = r.production_info_id
INNER JOIN requestor_info ri ON ri.id = r.requestor_info_id
INNER JOIN consumer c ON c.label = :consumer
WHERE ar.id > :lastSeen AND ar.consumer_id = c.id
LIMIT :maxRequests;"""

    class RequestCreator(object):
      def __init__ (self, api, creatorScript):
        self.__api = api
        self.__creatorScript = creatorScript
      def user (self):
        try:
          return securityApi.getUsernameFromID(self.owner_id)
        except KeyError:
          return "Unknown"
      def workflowspecs (self):
        return [self.createWorkflow ()]

      def createWorkflow (self):
        #FIXME: get cached request spec if necessary. 
        results = self.__api.getWorkflowSpecInfo(self.id)
        w, r = os.popen2 (self.__creatorScript)
        w.write(repr(results))
        w.close()
        return r.read()
      def locations (self):
        GET_REQUEST_LOCATIONS = """ 
        """
        #FIXME: get the correct location from the DB.
        return [{"name": "Any"}]

    class RequestCreatorFactory (object):
      def __init__(self, api, creatorScript=getenv("PR_WORKFLOW_CREATION_WRAPPER")):
        if not creatorScript:
          creatorScript="cat"
        self.__api = api
        self.__creatorScript = creatorScript
      def __call__(self):
        return RequestCreator (self.__api, self.__creatorScript)

    requests = self.apiPR.extendObjectWithData(["events", "approval_id", "id", "owner_id", "priority_hint"],
                                               GET_REQUEST_SPECS_SLICE,
                                               {"consumer": consumer,
                                                "lastSeen": lastSeen, 
                                                "maxRequests": maxRequests}, 
                                               RequestCreatorFactory(self.apiPR))
    print requests
    return {'requests': requests}

    
DeclarePlugin ("/Controllers/ProdRequest/Core", ProdRequest, {"baseUrl": "/ProdRequest"})

FrameworkApplication (label="ProdRequest",
                      name="ProdRequest",
                      url="../ProdRequest/initSession",
                      image="../Studio/images/mini.gif")

