from Framework import rpccall, xmlBooleanResponse, templatepage, templateXML, exposeSerialized
from Framework import Controller
from Framework.Utilities import selectFromDict
from Framework.Controller import require_args
from Tools.SecurityModuleCore import is_authorized, Role, Group
from cherrypy import expose
from ProdCommon.DataMgmt.DBS.DBSReader import DBSReader

from Modules.Utilities.Serializers.XML import PythonDictSerializer, PythonListSerializer
from Framework.PluginManager import DeclarePlugin

class ProdRequestDBSCore (object):
    def initDBSCgiApi (self, address):
        self.__url = address
        self.__api = DBSReader (self.__url)
    url = property (lambda self : self.__url, None, None)
    api = property (lambda self : self.__api, None, None)
        

class ProdRequestDBS (Controller):
    def __init__ (self, context):
        Controller.__init__ (self, context, __file__)
        self.core = ProdRequestDBSCore ()
        context.addService (self.core)
        self.core.initDBSCgiApi ("http://cmsdbsprod.cern.ch/cms_dbs_prod_global/servlet/DBSServlet")

    @expose
    def index (self):
        """Returns the version of the module."""
        return "ProdRequestDBS v0.1"
    
    @templateXML
    def getDatasetsInfo ():
        return {}
    
    @templateXML
    def getDatasetsByQuery (self, query="*", **args):
        datasets = self.core.api.listPrimaryDatasets ("*" + query + "*")
        limit = len (datasets)
        if args.has_key ("limit"):
            limit = int(args["limit"])
        return {'datasets': datasets[0:limit]}
    
    @templateXML
    @require_args ("primary_dataset")
    def getProcessedDatasets (self, primary_dataset, **args):
        datasetInfos = self.core.api.matchProcessedDatasets (primary_dataset, "*", "*")
        datasets = []
        for infoEntry in datasetInfos:
            for pathName in infoEntry["PathList"]:
                datasets.append (pathName)
        return {'datasets': datasets}
        
    @xmlBooleanResponse
    @require_args ("dbsAddress")
    @is_authorized (Role ("Admin"), Group ("ProdRequest"), onFail=bool)
    def setDBSAddress (self, dbsAddress, **args):
        self.core.initDBSCgiApi (dbsAddress)
        return True
    
    @exposeSerialized (serializer=PythonDictSerializer ("dbsurl"))
    def getDBSAddress (self, **args):
        return {"url": self.core.url}
        
DeclarePlugin ("/Controllers/ProdRequest/DBSInterface", ProdRequestDBS, {"baseUrl": "/ProdRequestDBS"})