CREATE TABLE IF NOT EXISTS approved_request (
  id INTEGER PRIMARY KEY,
  request_id INT CONSTRAINT request_id_exists REFERENCES request(id), 
  consumer_id INT CONSTRAINT consumer_id_exists REFERENCES consumer (id)
);

CREATE TABLE IF NOT EXISTS architecture (
  id INTEGER PRIMARY KEY,
  name TEXT
);

INSERT OR IGNORE INTO architecture (id, name) VALUES(1,"slc3_ia32_gcc323");
INSERT OR IGNORE INTO architecture (id, name) VALUES(2,"slc4_ia32_gcc345");
INSERT OR IGNORE INTO architecture (id, name) VALUES(3,"slc4_amd64_gcc345");

CREATE TABLE IF NOT EXISTS executable (
    id INTEGER PRIMARY KEY,
    name TEXT    
);

CREATE TABLE IF NOT EXISTS generator_info (
    id INTEGER PRIMARY KEY,
    executable TEXT,
    generator TEXT,
    cfg TEXT
);

CREATE TABLE IF NOT EXISTS dbs_server (
  id INTEGER PRIMARY KEY,
  url TEXT
);

INSERT OR IGNORE INTO dbs_server (id, url) VALUES (0, "http://cmsdbs.cern.ch/cms/prod/comp/DBS/CGIServer/prodquery?instance=%(instance)s");

CREATE TABLE IF NOT EXISTS dbs_instance (
  id INTEGER PRIMARY KEY,
  dbs_server_id INT CONSTRAINT dbs_server_id_exists REFERENCES dbs_server(id),
  instance TEXT
);

INSERT OR IGNORE INTO dbs_instance (id, dbs_server_id, instance) VALUES(0, 0, "MCGlobal/Writer");
INSERT OR IGNORE INTO dbs_instance (id, dbs_server_id, instance) VALUES(1, 0, "MCLocal_2/Writer");
INSERT OR IGNORE INTO dbs_instance (id, dbs_server_id, instance) VALUES(2, 0, "MCLocal_3/Writer");
INSERT OR IGNORE INTO dbs_instance (id, dbs_server_id, instance) VALUES(3, 0, "MCLocal_4/Writer");
INSERT OR IGNORE INTO dbs_instance (id, dbs_server_id, instance) VALUES(4, 0, "MCLocal_5/Writer");
INSERT OR IGNORE INTO dbs_instance (id, dbs_server_id, instance) VALUES(5, 0, "MCLocal_6/Writer");
INSERT OR IGNORE INTO dbs_instance (id, dbs_server_id, instance) VALUES(6, 0, "MCLocal_9/Writer");
INSERT OR IGNORE INTO dbs_instance (id, dbs_server_id, instance) VALUES(7, 0, "MCLocal_1/Writer");
INSERT OR IGNORE INTO dbs_instance (id, dbs_server_id, instance) VALUES(8, 0, "MCLocal_10/Writer");

CREATE TABLE IF NOT EXISTS location (
    id INTEGER PRIMARY KEY,
    name TEXT,
    local_dbs_id INT CONSTRAINT local_dbs_id_exists REFERENCES dbs_instance(id),
    user_group INTEGER
);

INSERT OR IGNORE INTO location (id, name, local_dbs_id, user_group) VALUES(1 << 1, "Any", 1, 0);
INSERT OR IGNORE INTO location (id, name, local_dbs_id, user_group) VALUES(1 << 2, "LCG team 1", 2, 59);
INSERT OR IGNORE INTO location (id, name, local_dbs_id, user_group) VALUES(1 << 3, "LCG team 2", 3, 60);
INSERT OR IGNORE INTO location (id, name, local_dbs_id, user_group) VALUES(1 << 4, "LCG team 3", 4, 61);
INSERT OR IGNORE INTO location (id, name, local_dbs_id, user_group) VALUES(1 << 5, "LCG team 4", 5, 62);
INSERT OR IGNORE INTO location (id, name, local_dbs_id, user_group) VALUES(1 << 6, "LCG team 5", 6, 63);
INSERT OR IGNORE INTO location (id, name, local_dbs_id, user_group) VALUES(1 << 7, "LCG team 6", 7, 64);
INSERT OR IGNORE INTO location (id, name, local_dbs_id, user_group) VALUES(1 << 8, "OSG team 1", 2, 65);
INSERT OR IGNORE INTO location (id, name, local_dbs_id, user_group) VALUES(1 << 9, "LCG team 7", 8, 66);
INSERT OR IGNORE INTO location (id, name, local_dbs_id, user_group) VALUES(1 << 10, "OSG team 2", 2, 67);
INSERT OR IGNORE INTO location (id, name, local_dbs_id, user_group) VALUES(1 << 11, "OSG", 1, 65);
INSERT OR IGNORE INTO location (id, name, local_dbs_id, user_group) VALUES(1 << 12, "ARC", 1, 0);
INSERT OR IGNORE INTO location (id, name, local_dbs_id, user_group) VALUES(1 << 13, "ARC2", 1, 70);

CREATE TABLE IF NOT EXISTS payload_status (
    id INTEGER PRIMARY KEY,
    status TEXT
);

INSERT OR IGNORE INTO payload_status (id, status) VALUES(0,"OPEN");
INSERT OR IGNORE INTO payload_status (id, status) VALUES(1,"CLOSED");

CREATE TABLE IF NOT EXISTS payload (
    id INTEGER PRIMARY KEY,
    name TEXT,
    parameterset TEXT,
    parsed_parameterset TEXT,
    pset_hash TEXT,
    version TEXT,
    creation_time TIMESTAMP,
    status INT,
    owner_id INT
);

CREATE TABLE IF NOT EXISTS cmssw (
    id INTEGER PRIMARY KEY,
    version TEXT,
    architecture_id INT CONSTRAINT architecture_id_exists REFERENCES architecture(id) 
);

INSERT OR IGNORE INTO cmssw (id, version, architecture_id) VALUES(0,"CMSSW_1_2_3", 1);


CREATE TABLE IF NOT EXISTS request_spec (
    id INTEGER PRIMARY KEY,
    text TEXT,
    pset_hash TEXT
);

CREATE TABLE IF NOT EXISTS entry_priority_hint (
    id INTEGER PRIMARY KEY,
    label TEXT
);

INSERT OR IGNORE INTO entry_priority_hint (id,label) VALUES(0,"LOW");
INSERT OR IGNORE INTO entry_priority_hint (id,label) VALUES(1,"NORMAL");
INSERT OR IGNORE INTO entry_priority_hint (id,label) VALUES(2,"HIGH");

CREATE TABLE IF NOT EXISTS event_type (
  id INTEGER PRIMARY KEY,
  label TEXT
);

INSERT OR IGNORE INTO event_type (id, label) VALUES(0,"INJECTED");
INSERT OR IGNORE INTO event_type (id, label) VALUES(1,"PENDING");
INSERT OR IGNORE INTO event_type (id, label) VALUES(2,"APPROVED");
INSERT OR IGNORE INTO event_type (id, label) VALUES(3,"FETCHED");
INSERT OR IGNORE INTO event_type (id, label) VALUES(4,"LOCALSUBMIT");
INSERT OR IGNORE INTO event_type (id, label) VALUES(5,"REJECTED");
INSERT OR IGNORE INTO event_type (id, label) VALUES(6,"RUNNING");
INSERT OR IGNORE INTO event_type (id, label) VALUES(7,"DONE");
INSERT OR IGNORE INTO event_type (id, label) VALUES(8,"RETIRED");
INSERT OR IGNORE INTO event_type (id, label) VALUES(9,"APPROVED_PRODMANAGER");


CREATE TABLE IF NOT EXISTS physics_group (
    id INTEGER PRIMARY KEY,
    name TEXT
);

INSERT OR IGNORE INTO physics_group (id, name) VALUES(0,"Individual");
INSERT OR IGNORE INTO physics_group (id, name) VALUES(1,"Higgs");
INSERT OR IGNORE INTO physics_group (id, name) VALUES(2,"SUSYBSM");
INSERT OR IGNORE INTO physics_group (id, name) VALUES(3,"EWK");
INSERT OR IGNORE INTO physics_group (id, name) VALUES(4,"Top");
INSERT OR IGNORE INTO physics_group (id, name) VALUES(5,"QCD");
INSERT OR IGNORE INTO physics_group (id, name) VALUES(6,"Diffraction");
INSERT OR IGNORE INTO physics_group (id, name) VALUES(7,"OnSel");
INSERT OR IGNORE INTO physics_group (id, name) VALUES(8,"Bphys");
INSERT OR IGNORE INTO physics_group (id, name) VALUES(9,"Muons");
INSERT OR IGNORE INTO physics_group (id, name) VALUES(10,"Egamma");
INSERT OR IGNORE INTO physics_group (id, name) VALUES(11,"JetMet");
INSERT OR IGNORE INTO physics_group (id, name) VALUES(12,"PFlowTau");
INSERT OR IGNORE INTO physics_group (id, name) VALUES(13,"Btag");

CREATE TABLE IF NOT EXISTS consumer (
    id INTEGER PRIMARY KEY,
    label TEXT
);

INSERT OR IGNORE INTO consumer (id, label) VALUES(0,"OPERATOR");
INSERT OR IGNORE INTO consumer (id, label) VALUES(1,"PRODMANAGER");

CREATE TABLE IF NOT EXISTS request_type (
    id INTEGER PRIMARY KEY,
    label TEXT
);

INSERT OR IGNORE INTO request_type (id,label) VALUES(0,"GEN+SIM");
INSERT OR IGNORE INTO request_type (id,label) VALUES(1,"DIGI+RECO");
INSERT OR IGNORE INTO request_type (id,label) VALUES(2,"AOD");
INSERT OR IGNORE INTO request_type (id,label) VALUES(3,"cmsRun");
INSERT OR IGNORE INTO request_type (id,label) VALUES(4,"cmsGen");

CREATE TABLE IF NOT EXISTS request_category (
    id INTEGER PRIMARY KEY,
    label TEXT
);

INSERT OR IGNORE INTO request_category (id, label) VALUES(0,"mc");
INSERT OR IGNORE INTO request_category (id, label) VALUES(1,"CSA08");

CREATE TABLE IF NOT EXISTS prodcommon (
    id INTEGER PROMARY KEY,
    label TEXT
);

INSERT OR IGNORE INTO prodcommon (id,label) VALUES(0, "PRODCOMMON_0_3_X");
INSERT OR IGNORE INTO prodcommon (id,label) VALUES(1, "PRODCOMMON_0_3_X_SLC4");

CREATE TABLE IF NOT EXISTS cmssw_to_prodcommon_rules (
    re TEXT,
    prodcommon_id INT CONSTRAINT prodcommon_id_exists REFERENCES prodcommon(id)
);
-- FIXME: Rename to singular.
INSERT OR IGNORE INTO cmssw_to_prodcommon_rules (re, prodcommon_id) VALUES(".*", 1);
INSERT OR IGNORE INTO cmssw_to_prodcommon_rules (re, prodcommon_id) VALUES("CMSSW_[1]_[5-7]_.*", 1);
INSERT OR IGNORE INTO cmssw_to_prodcommon_rules (re, prodcommon_id) VALUES("CMSSW_[1]_[0-3]_.*", 1);
INSERT OR IGNORE INTO cmssw_to_prodcommon_rules (re, prodcommon_id) VALUES("CMSSW_[0]_[0-9]_.*", 0);

CREATE TABLE IF NOT EXISTS processed_dataset (
  id INTEGER PRIMARY KEY,
  name TEXT,
  UNIQUE (name) ON CONFLICT IGNORE
);

CREATE TABLE IF NOT EXISTS primary_dataset (
  id INTEGER PRIMARY KEY,
  name TEXT,
  UNIQUE (name) ON CONFLICT IGNORE
);

CREATE TABLE IF NOT EXISTS requestor_info (
  id INTEGER PRIMARY KEY,
  physics_group_id INT CONSTRAINT physics_group_id_exists REFERENCES physics_group(id) DEFAULT 0, 
  owner_id INT
);

CREATE TABLE IF NOT EXISTS cfg (
  id INTEGER PRIMARY KEY,
  parameterset TEXT,
  UNIQUE (parameterset) ON CONFLICT IGNORE
);

CREATE TABLE IF NOT EXISTS physics_info (
  id INTEGER PRIMARY KEY,
  primary_dataset_id INT CONSTRAINT primary_dataset_id_exists REFERENCES primary_dataset(id),
  selection_efficiency FLOAT DEFAULT 1.0,
  cross_section FLOAT DEFAULT 0.0,  
  cfg_id INT CONSTRAINT cfg_id_exists REFERENCES cfg(id),
  request_type_id INT CONSTRAINT request_type_id_exists REFERENCES request_type(id) 
);

CREATE TABLE IF NOT EXISTS production_location (
  request_id INT CONSTRAINT request_id_exists REFERENCES request(id),
  location_id INT CONSTRAINT location_id_exists REFERENCES location(id)
);

CREATE TABLE IF NOT EXISTS production_info (
  id INTEGER PRIMARY KEY,
  priority_hint INT DEFAULT 50,
  requested_events INT,
  prodcommon_id INT CONSTRAINT prodcommon_id_exists REFERENCES prodcommon(id), 
  preferred_consumer_id INT CONSTRAINT preferred_consumer_id_exists REFERENCES consumer(id) DEFAULT 0, 
  cmssw_id INT CONSTRAINT cmssw_id_exists REFERENCES cmssw(id),
  request_category_id INT CONSTRAINT request_category_id_exists REFERENCES request_category(id)
);

CREATE TABLE IF NOT EXISTS history_log (
  id INTEGER PRIMARY KEY,
  event_type_id INT CONSTRAINT event_type_id REFERENCES event_type(id),  
  request_id INT CONSTRAINT request_id_exists REFERENCES request(id),
  time TIMESTAMP
);

CREATE TABLE IF NOT EXISTS split_type (
  id INTEGER PRIMARY KEY,
  label TEXT
);

INSERT INTO split_type (label) 
VALUES ("event");


CREATE TABLE IF NOT EXISTS processing_info (
  id INTEGER PRIMARY KEY,
  input_dataset_id INT CONSTRAINT input_dataset_id_exists REFERENCES processed_dataset(id),
  split_size INT DEFAULT 500, 
  split_type_id INT CONSTRAINT split_type_id_exists REFERENCES split_type(id)
);

CREATE TABLE IF NOT EXISTS pileup_info (
  id INTEGER PRIMARY KEY,
  pileup_dataset_id INT CONSTRAINT pileup_dataset_id_exists REFERENCES processed_dataset(id),
  files_per_job INT DEFAULT 1
);


CREATE TABLE IF NOT EXISTS request (
  id INTEGER PRIMARY KEY,
  requestor_info_id INT CONSTRAINT requestor_info_id_exists REFERENCES requestor_info(id),
  physics_info_id INT CONSTRAINT physics_info_id_exists REFERENCES physics_info(id),
  production_info_id INT CONSTRAINT production_info_id REFERENCES production_info(id),
  creation_log_entry_id INT CONSTRAINT creation_log_entry_id_exists REFERENCES history_log(id),
  current_log_entry_id INT CONSTRAINT current_log_entry_id_exists REFERENCES history_log(id),
  generator_info_id INT CONSTRAINT generator_info_id_exists REFERENCES generator_info(id), 
  request_spec_id INT CONSTRAINT request_spec_id_exists REFERENCES request_spec(id),
  pileup_info_id INT CONSTRAINT pileup_info_id_exists REFERENCES pileup_info(id),
  processing_info_id INT CONSTRAINT processing_info_id_exists REFERENCES processing_info(id),
  creation_hash INT
);
