CREATE TABLE IF NOT EXISTS request_view (
  id INTEGER PRIMARY KEY,
  label TEXT,
  owner_id INTEGER,
  query TEXT
);

INSERT INTO request_view 
(label, owner_id, query) VALUES ("*default*", -1, "/request/id/physics_info.primary_dataset.name/production_info.priority_hint");

CREATE TABLE IF NOT EXISTS column_type (
  id INTEGER PRIMARY KEY,
  label TEXT
);

INSERT INTO column_type (id, label) VALUES (1, "int");
INSERT INTO column_type (id, label) VALUES (2, "float");
INSERT INTO column_type (id, label) VALUES (3, "text");
INSERT INTO column_type (id, label) VALUES (4, "link");
INSERT INTO column_type (id, label) VALUES (5, "array");

CREATE TABLE IF NOT EXISTS column_description (
  id INTEGER PRIMARY KEY,
  label TEXT,
  field TEXT,
  type_id INT CONSTRAINT column_type_id_exists REFERENCES column_type(id)
);

INSERT INTO column_description 
(label, field, type_id) VALUES ("#", "id", 1);
INSERT INTO column_description 
(label, field, type_id) VALUES ("Input Dataset", "processing_info.input_dataset[processed_dataset].name", 3);
INSERT INTO column_description 
(label, field, type_id) VALUES ("Primary Dataset", "physics_info.primary_dataset.name", 3);
INSERT INTO column_description 
(label, field, type_id) VALUES ("Pileup Dataset", "pileup_info.pileup_dataset[processed_dataset].name", 3);
INSERT INTO column_description
(label, field, type_id) VALUES ("Requestes events", "production_info.requested_events", 1);
INSERT INTO column_description 
(label, field, type_id) VALUES ("Priority", "production_info.priority_hint", 1);
INSERT INTO column_description 
(label, field, type_id) VALUES ("CMSSW", "production_info.cmssw.version", 3);
INSERT INTO column_description 
(label, field, type_id) VALUES ("Request category", "production_info.request_category.label", 3);
INSERT INTO column_description 
(label, field, type_id) VALUES ("Selection efficiency", "physics_info.selection_efficiency", 2);
INSERT INTO column_description 
(label, field, type_id) VALUES ("Cross section", "physics_info.cross_section", 2);
INSERT INTO column_description 
(label, field, type_id) VALUES ("Request type", "physics_info.request_type.label", 3);
INSERT INTO column_description 
(label, field, type_id) VALUES ("Physics group", "requestor_info.physics_group.name", 3);
INSERT INTO column_description 
(label, field, type_id) VALUES ("Status", "current_log_entry[history_log].event_type.label", 3);



CREATE TABLE IF NOT EXISTS main_table_state (
  session_id TEXT,
  page INT,
  pageSize INT,
  query TEXT,
  textQuery TEXT,
  textQueryColumn TEXT,
  selected_request_id INT
);

INSERT INTO main_table_state (session_id, page, pageSize, query, textQuery, textQueryColumn, selected_request_id)
VALUES ('*default*', 0, 25, '/request/id/physics_info.primary_dataset.name/production_info.requested_events', '', 'Primary Dataset', -1);

CREATE TABLE IF NOT EXISTS query_operator (
  id TEXT,
  label TEXT,
  function TEXT
);

INSERT INTO query_operator
(id, label, function) VALUES (0, "is", "is");
INSERT INTO query_operator
(id, label, function) VALUES (1, "contains", "contains");
INSERT INTO query_operator
(id, label, function) VALUES (2, "starts with", "startswith");
INSERT INTO query_operator
(id, label, function) VALUES (3, "ends with", "endswith");
INSERT INTO query_operator
(id, label, function) VALUES (4, "is greater than", "gt");
INSERT INTO query_operator
(id, label, function) VALUES (5, "is less than", "lt");
INSERT INTO query_operator
(id, label, function) VALUES (6, "is greater or equal to", "lt");
INSERT INTO query_operator
(id, label, function) VALUES (7, "is less or equal to", "gt");

CREATE TABLE IF NOT EXISTS column_type_query_operators (
  column_type_id INT CONSTRAINT column_type_id_exist REFERENCES column_type(id),
  query_operator_id INT CONSTRAINT query_operator_id_exists REFERENCES query_operator(id)
);

INSERT INTO column_type_query_operators (column_type_id, query_operator_id)
VALUES (1,0);
INSERT INTO column_type_query_operators (column_type_id, query_operator_id)
VALUES (1,4);
INSERT INTO column_type_query_operators (column_type_id, query_operator_id)
VALUES (1,5);
INSERT INTO column_type_query_operators (column_type_id, query_operator_id)
VALUES (1,6);
INSERT INTO column_type_query_operators (column_type_id, query_operator_id)
VALUES (1,7);

INSERT INTO column_type_query_operators (column_type_id, query_operator_id)
VALUES (2,0);
INSERT INTO column_type_query_operators (column_type_id, query_operator_id)
VALUES (2,4);
INSERT INTO column_type_query_operators (column_type_id, query_operator_id)
VALUES (2,5);
INSERT INTO column_type_query_operators (column_type_id, query_operator_id)
VALUES (2,6);
INSERT INTO column_type_query_operators (column_type_id, query_operator_id)
VALUES (2,7);


CREATE TABLE IF NOT EXISTS query_state (
  session INTEGER PRIMARY KEY,
  field TEXT,
  operator_id INT,
  value TEXT
);

CREATE TABLE IF NOT EXISTS enabled_plugin (
  session TEXT,
  name TEXT,
  ordering INT
);

INSERT INTO enabled_plugin (session, name, ordering)
VALUES ("*default*", "MastHead", 0);
INSERT INTO enabled_plugin (session, name, ordering) 
VALUES ("*default*", "AvailableColumns", 0);
INSERT INTO enabled_plugin (session, name, ordering) 
VALUES ("*default*", "QueryBuilderHelper", 10);
INSERT INTO enabled_plugin (session, name, ordering) 
VALUES ("*default*", "QueryBuilder", 20);
INSERT INTO enabled_plugin (session, name, ordering) 
VALUES ("*default*", "MainTable", 30);
INSERT INTO enabled_plugin (session, name, ordering) 
VALUES ("*default*", "MainTableContents", 40);
INSERT INTO enabled_plugin (session, name, ordering)
VALUES ("*default*", "MainTablePager", 50);
INSERT INTO enabled_plugin (session, name, ordering)
VALUES ("*default*", "RequestStatusUpdater", 60);
INSERT INTO enabled_plugin (session, name, ordering)
VALUES ("*default*", "RequestEditor", 70);
INSERT INTO enabled_plugin (session, name, ordering)
VALUES ("*default*", "RequestApprover", 80);
INSERT INTO enabled_plugin (session, name, ordering)
VALUES ("*default*", "ErrorLog", 90);

CREATE TABLE IF NOT EXISTS session (
  session_id TEXT,
  time TIMESTAMP,
  owner_id TEXT
);

CREATE TABLE IF NOT EXISTS default_values (
  architecture TEXT,
  cmssw_version TEXT,
  priority_hint INT,
  preferred_consumer TEXT,
  physics_group TEXT,
  request_type TEXT,
  request_category TEXT
);

INSERT OR ROLLBACK INTO default_values (architecture, cmssw_version, 
                                  priority_hint, preferred_consumer, physics_group, request_type, request_category)
VALUES("slc4_ia32_gcc345", "CMSSW_2_0_3", 50, "PRODMANAGER", "Individual", "GEN+SIM", "mc");

CREATE TABLE session_error_log (
  id INTEGER PRIMARY KEY,
  session_id INT CONSTRAINT session_id_exists REFERENCES session(id),
  error_description_id INT CONSTRAINT error_description_id_exists REFERENCES error_description(id),
  user TEXT,
  time TIMESTAMP
);

CREATE TABLE IF NOT EXISTS error_description (
  id INTEGER PRIMARY KEY,
  name TEXT,
  description TEXT
);

INSERT INTO error_description (name, description)
VALUES("NO_ERROR", "");
INSERT INTO error_description (name, description)
VALUES("CANNOT_APPROVE", "You are not authorized to approve this request.");
INSERT INTO error_description (name, description)
VALUES("CANNOT_DENY", "You are not authorized to deny this request.");
INSERT INTO error_description (name, description)
VALUES("CANNOT_ARCHIVE", "You are not authorized to archive this request.");
INSERT INTO error_description (name, description)
VALUES("CANNOT_DELETE", "You are not authorized to delete this request.");
INSERT INTO error_description (name, description)
VALUES("CANNOT_CHANGE_STATUS", "You are not authorized to change the status of this request.");
