#ProdRequest stuff
from Models import WorkflowEntry, Payload, IdToPhysicsGroup

# NOTICE: this will work only with PRODCOMMON_0_2_x
from ProdCommon.MCPayloads.WorkflowMaker import WorkflowMaker
from ProdCommon.MCPayloads.WorkflowSpec import WorkflowSpec
from ProdCommon.MCPayloads.LFNAlgorithm import unmergedLFNBase, mergedLFNBase
from ProdCommon.CMSConfigTools.CfgInterface import CfgInterface
from ProdCommon.MCPayloads.DatasetExpander import splitMultiTier
import ProdCommon.MCPayloads.WorkflowTools as WorkflowTools
import ProdCommon.MCPayloads.UUID as MCPayloadsUUID
from ProdCommon.CMSConfigTools.CfgInterface import CfgInterface
from Models import SpecInfo
import popen2

def checkPSetType (pycfg):
    cfgInt = CfgInterface (pycfg, True)
    for outModName, val in cfgInt.outputModules.items(): 
        datasets = val.datasets() 
        for outDataset in datasets: 
            if outDataset.has_key ('dataTier'):
                return "NEW"
            else:
                return "OLD"
    return "OLD"

def createPSetHash(cfg):
    """
    _createPSetHash_

    Run the EdmConfigHash utility to create a PSet Hash for the
    cfg provided.

    An Exception will be raised if the command fails

    """
    pop = popen2.Popen3("EdmConfigHash", capturestderr=True)
    pop.tochild.write (cfg)
    pop.tochild.close ()
    pop.wait()
    exitStatus = pop.poll()
    if exitStatus:
        msg = "Error creating PSet Hash file:\n"
        msg += pop.childerr.read()
        raise RuntimeError, msg

    content = pop.fromchild.read()
    return content.strip()

def createPythonConfig(cfg):
    """
    _createPythonConfig_

    Generate a Python Config from the cfg provided.
    """
    pop = popen2.Popen3("EdmConfigToPython")
    pop.tochild.write (cfg)
    pop.tochild.close ()
    pycfg = pop.fromchild.read ()
    pop.wait()
    exitStatus = pop.poll()
    if exitStatus:
        msg = "Error creating Python cfg file:\n"
        msg += pop.childerr.read()
        raise RuntimeError, msg

    # TODO: make sure that the python object is valid.
    return pycfg 

    
def createRequestSpec (entry_id):
    entry = WorkflowEntry.get (entry_id)
    if entry.spec_info:
        return entry.spec_info.text
    
    reqReleaseName = entry.project.name
    workflowspecs = []
    payload = Payload.get (entry.payload.id)
    try:
        parsed_parameterset = createPythonConfig (payload.parameterset)
        if checkPSetType (parsed_parameterset) == "NEW":
            log ("Creating production workflow")
            log ("Input dataset: %s" % entry.primary_dataset)
            log ("CMSSW release: %s" % reqReleaseName)
            log ("PyCfg: %s" % payload.parameterset)
            if not reqReleaseName:
                reqReleaseName = payload.version
            
            maker = WorkflowMaker (entry.id, entry.primary_dataset, "Spring07")
            maker.setCMSSWVersion (reqReleaseName)
            maker.setPhysicsGroup (IdToPhysicsGroup (entry.physics_group))
            maker.setConfiguration (parsed_parameterset, Format="python", Type="string")
            maker.setPSetHash (createPSetHash (payload.parameterset))
        
            if not entry.pileup_dataset in ("", "None", None, "pileup_dataset", "undefined"):
                log ("Pileup dataset: %s" % entry.pileup_dataset)
                maker.addPileupDataset (entry.pileup_dataset)
            
            if not entry.input_dataset in ("", "None", None, "input_dataset", "undefined"):
                log ("Input dataset: %s" % entry.input_dataset)
                maker.addInputDataset (entry.input_dataset)
                maker.inputDataset['SplitType'] = "event"
                maker.inputDataset['SplitSize'] = 500
    
            maker.addSelectionEfficiency (entry.selection_efficiency)
    
            workflowspec = maker.makeWorkflow ()
            #TODO: add the same as above for output datasets.
        workflowspec = workflowspec.saveString ()
        entry.spec_info = SpecInfo (text=workflowspec)
    except Exception, e:
        workflowspec = "<ErrorInWorkflowCreation>%s</ErrorInWorkflowCreation>" % e
    return workflowspec

if __name__ == "__main__":
    print "<!--"
    # Standard stuff to include for ProdCommonPlugin
    from ProdRequestCommon.ProdCommonCore.WorkflowCreatorHelper import initWorkflowCreator, log

    options = initWorkflowCreator ()
    print "-->"
    
    workflowspec = createRequestSpec (options.entryId)

    print workflowspec
    
