# Standard stuff to include for ProdCommonPlugin
from ProdRequestCommon.ProdCommonCore.WorkflowCreatorHelper import initWorkflowCreator, log

# 
from Models import WorkflowEntry, IdToPhysicsGroup
from datetime import datetime

import ProdCommon.MCPayloads.WorkflowTools as WorkflowTools
from ProdCommon.MCPayloads.WorkflowMaker import WorkflowMaker
from ProdCommon.CMSConfigTools.ConfigAPI.CMSSWConfig import CMSSWConfig
from ProdCommon.MCPayloads.CmsGenWorkflowMaker import CmsGenWorkflowMaker
from Models import SpecInfo

from os import getenv, popen
import os

def createWorkflow (entryId):
    try:
        entry = WorkflowEntry.get (entryId)
        # Value for the workflow is cached
        if entry.spec_info:
            return entry.spec_info.text
        pwd = getenv ("PWD")
        oldCmsswSearchPath = getenv ("CMSSW_SEARCH_PATH")
        os.environ ["CMSSW_SEARCH_PATH"] = "%s:%s" % (pwd, oldCmsswSearchPath)
        # FIXME: ugly hack because FWCore does not support input from string.
        tmpCfgFilename = ("cfg"+entryId+ ".cfg").replace (" ","+")
        tmpCfgFile = file (tmpCfgFilename, "w")
        tmpCfgFile.write (entry.payload.parameterset)
        tmpCfgFile.close ()
        cfgFile = tmpCfgFilename
        
        # for the time being we support only cfg file. We might have to support multiple types soon
        cfgType = "cfg"
        if cfgType == "cfg":
            from FWCore.ParameterSet.Config import include
            cmsCfg = include(cfgFile) 
        else:
            modRef = imp.find_module( os.path.basename(cfgFile).replace(".py", ""),  os.path.dirname(cfgFile))
            cmsCfg = modRef.process
        
        cfgWrapper = CMSSWConfig()
        cfgWrapper.originalCfg = file(cfgFile).read()
        cfgInt = cfgWrapper.loadConfiguration(cmsCfg)
        cfgInt.validateForProduction()
    except Exception, e:
        return "<error>Unable to create request: %s</error>" % e
    
    #  //
    # // Instantiate a WorkflowMaker
    #//
    # FIXME: label should be taken from the DB.
    primary_dataset_name = "%s-%s" % (entry.primary_dataset, entry.generator_info.generator)
    maker = CmsGenWorkflowMaker (entry.id, primary_dataset_name, "CSA07" )
    
    maker.setCMSSWVersion(entry.project.name)
    maker.setPhysicsGroup(IdToPhysicsGroup (entry.physics_group))
    maker.setConfiguration(cfgWrapper, Type = "instance")
    maker.setCmsGenConfiguration(entry.generator_info.cfg)
    maker.setCmsGenParameters(generator=entry.generator_info.generator)

    #psethash=popen ("export old=$PWD; export SCRAM_ARCH=slc4_ia32_gcc345; cd /afs/cern.ch/cms/sw/slc4_ia32_gcc345/cms/cmssw/CMSSW_1_4_6; eval `scramv1 run -sh`; cd $old; cat %(cfgFile)s | EdmConfigHash" % locals()).read()
    psethash=WorkflowTools.createPSetHash(cfgFile)
    maker.setPSetHash(psethash)
    
    # FIXME: category hardcoded to "mc". Might be nice to get it from DB as well. 
    category = "mc"
    maker.changeCategory(category)
    
    if entry.selection_efficiency:
        maker.addSelectionEfficiency(entry.selection_efficiency)
        maker.addCmsGenSelectionEfficiency(entry.selection_efficiency)
    
    #  //
    # // Pileup sample?
    #//
    if not entry.pileup_dataset in (None, "pileup_dataset", "None", "undefined"):
        # FIXME: pileupFilesPerJob is hardcoded for the time being. Should really be in DB.
        pileupFilesPerJob = 1
        maker.addPileupDataset( entry.pileup_dataset, pileupFilesPerJob)

    if not entry.input_dataset in ("", "None", None, "input_dataset", "undefined"):
        log ("Input dataset: %s" % entry.input_dataset)
        maker.addInputDataset (entry.input_dataset)
        maker.inputDataset['SplitType'] = "event"
        maker.inputDataset['SplitSize'] = 500
    
    spec = maker.makeWorkflow()
    
    log ("Created: %s-Workflow.xml" % maker.workflowName)
    log ("From: %s " % cfgFile)
    log ("Output Datasets:")

    [ log (
         "/%s/%s/%s\n" % (
           x['PrimaryDataset'],
           x['ProcessedDataset'],
           x['DataTier'])) for x in spec.outputDatasets()]
    result = spec.saveString ()
    entry.spec_info = SpecInfo (text=result)
    return result        

if __name__ == "__main__":
    options = initWorkflowCreator ()
    print createWorkflow (options.entryId)
