from sqlalchemy import *
import re
import os
import random

def _simpleLog (string):
  print string

class _ConnectionTester (object):
  def __init__ (self, dbi, connectionType, 
                logMethod=_simpleLog, errorMethod=_simpleLog):
    self.__dbi = dbi
    self.__connectionType = connectionType
  
  def test(self):
    try:
      if self.__dbi.con == None: return False
      if self.__dbi.con == '': return False
      if self.__connectionType == "sqlite":
        return self.testSqliteConnection()
      elif self.__connectionType == "oracle":
        return self.testOracleConnection()
      else:
        self.log ("Unknown database type %s" % self.__connectionType)
        return False
    except:
      return False
  
  def testOracleConnection(self):
    try:
      result = self.__dbi.con.connect().execute( 'select * from dual' )
      assert result > 0
    except:
      return False
    return True
  
  def testSqliteConnection(self):
    try:
      curs = self.__dbi.con.cursor()
      curs.execute( 'select * from sqlite_master' )
      assert len(curs.fetchone()) > 0
      curs.close()
    except Exception, inst:
      return False
    return True

_sqliteOptions = {"convert_unicode": True, 
                  "encoding": 'utf-8', 
                  "pool_size": 1, 
                  "pool_recycle": 30}

_defaultOptions = {"convert_unicode": True, 
                   "encoding": 'utf-8', 
                   "pool_size": 5, 
                   "pool_recycle": 600}
                  
class DBApi (object):
  class DBIInfo (object):
    dbi = None
    dbType = None
    con = None

  def __init__ (self, 
                logMethod=lambda string : _simpleLog (string),
                debugMethod=lambda string : _simpleLog (string)):
    self.log = logMethod
    self.debug = debugMethod
    self.dbi = DBApi.DBIInfo ()

  def connect  (self, database, dbType):
    self.dbi.dbType = dbType
    # if database was read from the file and doesn't had any sqlite:/// prefix we must create it
    if self.dbi.dbType == 'sqlite':
       if not 'sqlite:///' in database:
          database="sqlite:///" + database
    try:
      #self.context.Logger().message ("Creating engine for %s" % database)
      if 'sqlite:///' in database:
          self.dbi.con = create_engine(database, **_sqliteOptions)
      else:
          self.dbi.con = create_engine(database, **_defaultOptions)
    except Exception, e:
      self.log ("Could not connect to database engine")
      self.log (e)

  def close(self):
    self.debug ("Closing database connection")
    if self.testConnection () == True:
      self.dbi.con.dispose ()
    self.debug ("Database connection closed")

  def extendObjectWithData (self, fieldnames, sqlstmt, binds, cls):
    if type(fieldnames) == str:
      fieldnames = [fieldnames]
    result = self.getDataTable (sqlstmt, binds)
    dataObj = []

    cellsRange = range (0, len (fieldnames))
    for row in result:
      item = cls()
      i = 0
      for f in fieldnames:
        setattr (item, f, row[i])
        i += 1
      dataObj.append (item)
    return dataObj

  class EmptyExstensibleObject (object):
    pass
    
  def getDataPythonObject (self, fieldnames, sqlstmt, binds):
    self.extendObjectWithData (self, fieldnames, sqlstmt, 
                               binds, EmptyExstensibleObject)

  def getDataObject(self, fieldnames, sqlstmt, binds = None):
    if type(fieldnames) == str:
      fieldnames = [fieldnames]
    result = self.getDataTable (sqlstmt, binds)
    dataObj = []

    cellsRange = range (0, len (fieldnames))
    for row in result:
      item = {}
      i = 0
      for f in fieldnames:
        item[f] = row[i]
        i += 1
      dataObj.append (item)
    return dataObj

  def getDataTable(self, sqlstmt, binds=None):
    conn = self.dbi.con.connect()
    try:
      if binds == None:
          return conn.execute(sqlstmt)
      else:
          return conn.execute(sqlstmt, binds)
    except Exception, e:
      self.log(e)
      self.log(sqlstmt)
      if binds: self.log(binds)
      raise e

  def editDataObject(self, statements, bindsList = None):
    "Can take either a single statement or a list of statements and binds"
    if type (statements) != list:
      statements = [statements]
    if type (bindsList) != list:
      bindsList = [bindsList]
    assert (len(bindsList) == len (statements))

    connection = self.dbi.con.connect()
    trans = connection.begin()
    try:
      for i in range (0, len (statements)):
        connection.execute(statements.pop (0), bindsList.pop (0))
      trans.commit()
    except Exception, e:
      print e
      trans.rollback()
      self.log (statements)
      self.log (bindsList)
      self.log ("Couldn't commit: %s" % e)
      raise e

def _makeBindName (key, iteration):
  splitted = key.split (".")
  bindName = splitted[len(splitted) - 1]
  if iteration == 0:
    return bindName
  return bindName + str (iteration+1)

def _onlyLastTable (fieldName):
  parts = fieldName.split (".")
  if len (parts) <= 2:
    return fieldName
  return ".".join (parts[len(parts)-2:])

def _parseAlias (fieldName):
  nameDef = fieldName.strip("]").split("[")
  name = nameDef[0]
  if len(nameDef) > 1:
    return name, nameDef[1]
  return name, name
  

class QueryMaker (object):
  def __init__ (self, mainTable, innerJoin=True):
    self.__joinStatement = (innerJoin and "INNER JOIN") or "LEFT OUTER JOIN"
    self.__mainTable = mainTable
    self.__joinedTables = []
    self.__fields = []
    self.__conditions = []
    self.__bindVariables = {}
    self.__bindings = {}
    self.__ordering = []
    self.__tableAliases = {}

  def joinTable (self, joinedTable, initialTable=None):
    initialTable = initialTable or self.__mainTable
    assert ('.' not in joinedTable)
    assert ('.' not in initialTable)
    if (joinedTable, initialTable) not in self.__joinedTables:
      self.__joinedTables.append ((joinedTable, initialTable))

  def addField (self, name):
    """ Adds a given field to a table. In case the field is actually in another
        table, recursively join the tables.
    """
    table = self.__mainTable
    parts = name.split (".")
    for i in range (0, len (parts)):
      name, alias = _parseAlias(parts.pop(0))
      self.addAlias (name, alias)
      possibleField = "%s.%s" % (table, name)
      if name == self.__mainTable:
        continue
      elif not len (parts):
        if possibleField not in self.__fields:
          self.__fields.append (possibleField)
      else:
        self.joinTable (name, table)
      table = name

  def addAlias (self, alias, real):
    self.__tableAliases[alias] = real

  def makeBindVariables (self, key, value, op, iteration=0):
    bindName = _makeBindName (key, iteration)
    if _makeBindName (key, iteration) in self.__bindVariables:
      return self.makeBindVariables (key, value, op, iteration+1)
    self.__bindVariables[bindName] = (key, op)
    if op in "contains":
      value = "%%%s%%" % value
    if op in "beginswith":
      value = "%%%s" % value
    if op in "endswith":
      value = "%s%%" % value
    self.__bindings[bindName] = value
    return (bindName, key, value)

  def bindings (self):
    return self.__bindings
  
  def joinedTables (self):
    return self.__joinedTables
  
  def fields (self):
    return self.__fields

  def addSimpleCondition (self, key, value):
    self.addCondition (key, value, "is")

  def addConditionByString (self, conditionString):
    match = re.match ("(.*)\\((.*),(.*)\\)", conditionString)
    if not match:
      print conditionString
      assert (False)
    groups = match.groups ()
    op =  groups[0]
    fieldName = groups[1]
    value = groups[2]
    table = self.__mainTable
    parts = fieldName.split ('.')
    
    for i in xrange(len(parts)):
      name, alias = _parseAlias (parts.pop (0))
      self.addAlias (name, alias)
      possibleField = "%s.%s" % (table, name)
      if name == self.__mainTable:
        continue
      elif not len (parts):
        self.makeBindVariables (possibleField, value, op)
      else:
        self.joinTable (name, table)
      table = name
    
  def addCondition (self, key, value, op):
    table = self.__mainTable
    parts = key.split(".")
    for i in xrange(len(parts)):
      part = parts.pop (0)
      possibleField = "%s.%s" % (table, part)
      if part == self.__mainTable:
        continue
      elif not len(parts):
        self.makeBindVariables(possibleField, value, op)
      else:
        self.joinTable(part, table)
      table = part
    
  def addOrdering (self, key, asc=True):
    if key in self.__fields and key not in self.__ordering:
      self.__ordering.append ((key, asc))

  def _buildCondition (self, field, bindname, op):
    opsMap = {"is": "%s = :%s",
              "isnot": "%s != :%s",
              "contains": "%s LIKE :%s",
              "beginswith": "%s LIKE :%s",
              "endswith": "%s LIKE :%s",
              "gt": "%s > :%s",
              "lt": "%s < :%s"}
    return opsMap[op] % (_onlyLastTable(field), bindname)

  def getTableRealName (self, alias):
    if not self.__tableAliases.has_key (alias):
      return alias
    return self.__tableAliases[alias]

  def createQuery (self, mixer="AND", limit="", offset="", operation="%s"):
    mixer = " " + mixer + " "
    joinString = self.__joinStatement + " %(joinedTable)s %(alias)s on %(mainTable)s.%(alias)s_id = %(alias)s.id";
    joins = "\n".join ([joinString %  {"joinedTable": self.getTableRealName (joinedTable),
                                       "alias": joinedTable,
                                       "mainTable": initialTable}
                        for joinedTable, initialTable in self.__joinedTables])
    conditions = mixer.join ([self._buildCondition (field, bindname, op) 
                              for bindname, (field, op) in self.__bindVariables.items ()])
    if conditions:
      conditions = "where " + conditions
    limit = limit and ("limit %s" % limit)
    offset = offset and ("offset %s" % offset)
      
    ordering = ""
    if self.__ordering:
      ordering = "order by %s" % ",".join (["%s %s" % (key, ((asc and "ASC") or "DESC"))
                                             for key, asc in self.__ordering]) 
    return re.sub("\n[\n]+", "\n", 
                  "select %s from %s\n%s\n%s\n%s\n%s\n%s;" % (", ".join ([operation % field for field in self.__fields]), 
                  self.__mainTable,
                  joins, conditions, ordering, limit, offset)).replace ("\n;", ";")

def buildQueryFromUrl (queryUrl):
  """ Generic way of querying the prodrequets database.
      The syntax of the query is
    
      /<main-table>/<field 1>[(<alias 1>)]/.../<field n>[(<alias n>)]
      ?q=<cond 1>,<cond-2>...
      &o=<order by column 1>,<order by column 2>
      &l=<limit>
      &off=<offset>
      &op=<operation>
      
      For example:
      
      /requests/id/primary_dataset.name/production_info.requested_events
      ?q=contains(output_dataset.name,foo);is(input_dataset.name,bar)
      &o=production_info.requested_events
      &l=10
      &off=20
      
      will show the id, name of the primary dataset, number of requested events,
      for entries [20-29] of all the requests whose output_dataset.name contains
      "foo" and whose input_dataset contains (foo). 
  """
  path, opts = queryUrl.strip ("/").split ("?")
  pathItems = path.split ("/")
  optsItems = opts.split ("&")
  mainTable = pathItems.pop (0)
  maker = QueryMaker (mainTable, False)
  for field in pathItems:
    maker.addField (field)
    
  limit = ""; offset=""; op="%s"
  for option in optsItems:
    optLabel, value = option.split ("=")
    if optLabel == "q":
      for condition in value.strip(";").split (";"): 
        maker.addConditionByString (condition)
    if optLabel == "o":
      for ordering in value.split (","):
        maker.addOrdering (ordering)
    if optLabel == "l":
      limit = value
    if optLabel == "off":
      offset = value
    if optLabel == "op":
      op = value
  return (maker.fields (), maker.createQuery (limit=limit, offset=offset, operation=op), maker.bindings ())


class InsertionMaker (object):
  def __init__ (self, table):
    self.table = table

  def insertByQuery (self, query):
    joins = ["%s_id" % t for t in _query.joinedTables ()]
    return "INSERT INTO %s\n(%s)\n%s" % (self.table,
                                       ", ".join (joins),
                                       query.createQuery ())
  
  def insertByValues (self, **kwds):    
    columns = []
    values = []
    for key, value in kwds.iteritems ():
      if type(value) == str:
        value = re.escape (value)
      columns.append (key)
      values.append (value)
    return "INSERT INTO %s\n(%s)\nVALUES(%s)" % (table, 
                                               ", ".join (columns), 
                                               ", ".join (values))

import md5

class BindsBuilder (object):
  def __init__ (self, values):
    self.__values = values
    
  def __call__ (self, *binds):
    if type (binds) == str:
      binds = [binds]
    binds.extend (self.__defaults)
    return {}([(key, self.__values[key]) for key in binds])
  
  def __get__ (self, key):
    return self.__value [key]
  
  def add (self, key, value):
    self.__values[key] = value

  def default (self, keys):
    if type (keys) != list:
      keys = [keys]
    self.__defaults = keys

class ProdRequestGUIApi (DBApi):
  def __init__(self):
    DBApi.__init__(self)
  
  def cloneSession (self, oldSessionId, newSessionId, newOwnerId):
    """ Clones an already existing session
    """
    args = ArgSelector(locals());
    CREATE_SESSION = """INSERT INTO session (session_id, time, owner_id)
                            VALUES (:newSessionId, date('now'), :newOwnerId);"""

    CLONE_PLUGIN_LIST = """INSERT INTO enabled_plugin (session, name)
                           SELECT :newSessionId, ep.name 
                           FROM enabled_plugin ep 
                           WHERE ep.session = :oldSessionId"""
                           
    CLONE_TABLE_INFO = """INSERT INTO main_table_state
      (session_id, page, pageSize, query, textQuery, textQueryColumn)
      SELECT :newSessionId, d_mts.page, d_mts.pageSize, d_mts.query, d_mts.textQuery, d_mts.textQueryColumn
      FROM main_table_state d_mts
      WHERE d_mts.session_id = :oldSessionId;"""
    
    self.editDataObject ([CREATE_SESSION, 
                          CLONE_PLUGIN_LIST, 
                          CLONE_TABLE_INFO],
                          [args("oldSessionId", "newSessionId", "newOwnerId"),
                           args("oldSessionId", "newSessionId"),
                           args("oldSessionId", "newSessionId")])
  
  def createNewSession (self, session_id, owner_id):
    """ Inserts a new session and copies the default plugin into it
    """
    self.cloneSession('*default*', session_id, owner_id)
  
  def hasSession (self, session_id):
    HAS_SESSION = """SELECT count(session_id) 
                     FROM session WHERE session_id = :session_id
                     LIMIT 1"""
    result = self.getDataObject ("count", HAS_SESSION, {"session_id": session_id})
    return result[0]["count"] == 1 

  def checkSessionOwner(self, session_id, owner_id):
    CHECK_SESSION_OWNERSHIP = """SELECT count(session_id)
    FROM session 
    WHERE session_id = :session_id AND owner_id = :owner_id
    """
    args = ArgSelector(locals());
    results = self.getDataObject("count", CHECK_SESSION_OWNERSHIP, 
                                 args("session_id", "owner_id"))
    return results[0]["count"]
    
  def updateQueryConditions (self, session, conditions, textQuery, textQueryColumn):
    GET_QUERY = """SELECT query 
                   FROM main_table_state 
                   WHERE session_id = :session"""
    query = self.getDataObject ("query", GET_QUERY, 
                                {"session": session})[0]["query"]
    fields = query.split ("?")[0]
    query = "%s?%s" % (fields, conditions)
    UPDATE_SESSION_QUERY = """UPDATE main_table_state 
                              SET query = :query , textQuery = :textQuery, textQueryColumn = :textQueryColumn, page = 0
                              WHERE session_id = :session""" 
    self.editDataObject(UPDATE_SESSION_QUERY, 
                        {"query": query, "session": session, 
                         "textQuery": textQuery, "textQueryColumn": textQueryColumn})

  def updateTextQueryField (self, session, newLabel):
    GET_QUERY_AND_FIELD = """SELECT mts.query, cd.field, nl.field
                             FROM main_table_state mts
                             LEFT OUTER JOIN column_description cd ON cd.label = mts.textQueryColumn 
                             LEFT OUTER JOIN column_description nl ON nl.label = :newLabel
                             WHERE mts.session_id = :session"""
    print GET_QUERY_AND_FIELD
    print session
    result = self.getDataObject (["query", "oldField", "newField"], 
                                 GET_QUERY_AND_FIELD, 
                                 {"session": session, "newLabel": newLabel})[0]
    query = result["query"]
    oldField = result["oldField"]
    newField = result["newField"]
    try:
      fields, conditions = query.split ("?")
    except:
      fields, conditions = query, ""
    conditions = conditions.replace (oldField, newField)
    
    UPDATE_SESSION_QUERY = """UPDATE main_table_state 
                              SET query = :query , textQueryColumn = :textQueryColumn, page = 0
                              WHERE session_id = :session"""
    self.editDataObject (UPDATE_SESSION_QUERY, {"query": "%s?%s" % (fields, conditions),
                                                "session": session, 
                                                "textQueryColumn": newLabel})

  def addColumn (self, session, columnLabel):
    GET_QUERY_AND_TABLE = """SELECT mts.query, cd.field 
                   FROM main_table_state mts
                   INNER JOIN column_description cd ON cd.label = :columnLabel
                   WHERE mts.session_id = :session"""
                   
    results = self.getDataObject (["query", "fieldName"], GET_QUERY_AND_TABLE, 
                                  {"session": session,
                                   "columnLabel": columnLabel})[0]
    
    query = results["query"]
    fieldName = results["fieldName"]
    
    
    try:
      fields, conditions = query.split ("?")
    except:
      fields, conditions = query, ""
    
    if fieldName in fields:
      return

    query = "%s?%s" % (os.path.join (fields, fieldName), conditions)
    UPDATE_SESSION_QUERY = """UPDATE main_table_state 
                              SET query = :query
                              WHERE session_id = :session"""
    self.editDataObject (UPDATE_SESSION_QUERY, {"query": query,
                                                "session": session})

  def removeColumn (self, session, columnLabel):
    GET_QUERY_AND_TABLE = """SELECT mts.query, cd.field 
                   FROM main_table_state mts
                   INNER JOIN column_description cd ON cd.label = :columnLabel
                   WHERE mts.session_id = :session"""
  
    results = self.getDataObject (["query", "fieldName"], GET_QUERY_AND_TABLE, 
                                  {"session": session,
                                   "columnLabel": columnLabel})[0]
  
    query = results["query"]
    fieldName = results["fieldName"]
    try:
      fields, conditions = query.split ("?")
    except:
      fields, conditions = query, ""
    query = "%s?%s" % (fields.replace(fieldName, "").replace ("//","/").strip ("/"), conditions)
    UPDATE_SESSION_QUERY = """UPDATE main_table_state 
                              SET query = :query
                              WHERE session_id = :session"""
    self.editDataObject (UPDATE_SESSION_QUERY, {"query": query,
                                                "session": session})
    

  def selectRequest (self, session_id, request_id):
    SELECT_REQUEST = """UPDATE main_table_state 
                        SET selected_request_id = :request_id
                        WHERE main_table_state.session_id = :session_id;"""
    args = ArgSelector(locals());
    self.editDataObject (SELECT_REQUEST, args("session_id", "request_id"))

  def updateTablePage (self, session, page):
    UPDATE_TABLE_PAGE = """UPDATE main_table_state
                           SET page = :page
                           WHERE session_id = :session;"""
    self.editDataObject(UPDATE_TABLE_PAGE, {"page": page, "session": session})

  def setSessionError (self, session_id, user, errorName):
    SET_SESSION_ERROR = """INSERT INTO session_error_log (session_id, time, user, error_description_id)
                           SELECT :session_id, date('now'), :user, ed.id
                           FROM error_description ed
                           WHERE ed.name = :errorName;"""
    args = ArgSelector(locals())
    self.editDataObject(SET_SESSION_ERROR, args("session_id", "user", "errorName"))

  def cleanSessionError (self, session_id, user):
    self.setSessionError(session_id, user, "NO_ERROR")

class ArgSelector(object):
  def __init__ (self, argsDict):
    self.__argsDict = argsDict
  def __call__ (self, *keys):
    return dict([(key, self.__argsDict[key]) 
                  for key in keys
                  if key in self.__argsDict])

class ProdRequestApi (DBApi):
  def __init__ (self):
    DBApi.__init__ (self)

  def simpleInsert (self, table, **kwds):
    columns = []
    values = []
    bindings = {}
    for key, value in kwds.iteritems ():
      if type(value) == str:
        value = re.escape (value).replace ("\\_", "_")
      bindings[key] = value
    self.editDataObject ("INSERT INTO %s (%s) VALUES(%s)" % (table, 
        ", ".join ([key for key in kwds.iterkeys()]), 
        ", ".join ([":" + key for key in kwds.iterkeys()])), kwds)

  def queryByUrl (self, queryUrl):
    fields, query, binds = buildQueryFromUrl (queryUrl)
    return [list (row) for row in self.getDataTable (query, binds)]
  
  def logEvent (self, request_id, eventType):
    INSERT_LOG_ENTRY = """INSERT INTO history_log (event_type_id, request_id, time)
                          SELECT e.id, :request_id, date('now')
                          FROM event_type e
                          WHERE e.label = :eventType;"""
    UPDATE_REQUEST_CURRENT_EVENT = """UPDATE request 
                                      SET current_log_entry_id = last_insert_rowid()
                                      WHERE id = :request_id"""
    args = ArgSelector(locals())
    self.editDataObject([INSERT_LOG_ENTRY, 
                         UPDATE_REQUEST_CURRENT_EVENT], 
                        [args("request_id", "eventType"),
                         args("request_id")])
  
  def approveRequest (self, request_id, consumer):
    APPROVE_REQUEST = """INSERT INTO approved_request (request_id,consumer_id)
                         SELECT :request_id, c.id
                         FROM consumer c
                         WHERE c.label = :consumer"""
    args = ArgSelector(locals())
    self.editDataObject(APPROVE_REQUEST, args("request_id", "consumer"))
    self.logEvent(request_id, "APPROVED")
  
  # TODO: add a trigger that cleans up unneeded rows.
  def archiveRequest (self, request_id):
    self.logEvent(request_id, "RETIRED")

  def deleteRequest (self, request_id):
    DELETE_REQUEST = """DELETE FROM request WHERE id = :request_id;"""
    self.editDataObject(DELETE_REQUEST, {"request_id": request_id})
    self.logEvent(request_id, "RETIRED")

  def denyRequest (self, request_id):
    self.logEvent(request_id, "REJECTED")
  
  def getWorkflowSpecInfo (self, request_id):
    GET_WORKFLOW_SPEC_INFO = """/request
    /id
    /physics_info.primary_dataset.name
    /processing_info.input_dataset[processed_dataset].name
    /pileup_info.pileup_dataset[processed_dataset].name
    /creation_log_entry[history_log].time
    /current_log_entry[history_log].time
    /current_log_entry[history_log].event_type.label
    /production_info.requested_events
    /production_info.cmssw.version
    /production_info.cmssw.architecture.name
    /production_info.request_category.label
    /production_info.preferred_consumer[consumer].label
    /production_info.prodcommon.label
    /generator_info.executable
    /generator_info.generator
    /generator_info.cfg
    /physics_info.cross_section
    /physics_info.selection_efficiency
    /physics_info.request_type.label
    /physics_info.cfg.parameterset
    /requestor_info.physics_group.name?q=is(request.id,:id)""".replace ("\n", "")
    GET_WORKFLOW_SPEC_INFO = re.sub ("\s*/", "/", GET_WORKFLOW_SPEC_INFO)
    fields, query, bindings = buildQueryFromUrl (GET_WORKFLOW_SPEC_INFO)
    print query
    fields = [f.replace (".", "__") for f in fields]
    result = self.getDataObject (fields, query, {"id": request_id})
    if not result:
      return {}
    result = result[0]
    return result
  
  def getRequestInfo (self, request_id):
    """Helper method which returns the interesting parts about a request.
       Used for both the 'more information' GUI and for creating a requestspec.
    """
    # TODO: move the GET_CURRENT_REQUEST_INFO_URL to a file so that it can be 
    #       modified without restarting the server.
    GET_CURRENT_REQUEST_INFO_URL = """/request
    /id
    /physics_info.primary_dataset.name
    /processing_info.input_dataset[processed_dataset].name
    /pileup_info.pileup_dataset[processed_dataset].name
    /creation_log_entry[history_log].time
    /current_log_entry[history_log].time
    /current_log_entry[history_log].event_type.label
    /production_info.requested_events
    /production_info.cmssw.version
    /production_info.request_category.label
    /production_info.preferred_consumer[consumer].label
    /production_info.prodcommon.label
    /physics_info.cross_section
    /physics_info.selection_efficiency
    /physics_info.request_type.label
    /requestor_info.physics_group.name?q=is(request.id,:id)""".replace ("\n", "")
    GET_CURRENT_REQUEST_INFO_URL = re.sub ("\s*/", "/", GET_CURRENT_REQUEST_INFO_URL)
    fields, query, bindings = buildQueryFromUrl (GET_CURRENT_REQUEST_INFO_URL)
    fields = [f.replace (".", "__") for f in fields]
    result = self.getDataObject (fields, query, {"id": request_id})
    if not result:
      return {}
    result = result[0]
    return result
  
  def createPileupInfoQuery (self, kwds):
    PILEUP_INFO_DEFAULT_FILES_PER_JOB = """INSERT INTO pileup_info 
    (pileup_dataset_id)
    SELECT (pileup.id)
    FROM processed_dataset pileup
    WHERE pileup.name = :pileup_dataset
    """
    PILEUP_INFO = """INSERT INTO pileup_info
    (pileup_dataset_id, files_per_job)
    SELECT (pileup.id, :files_per_job)
    FROM processed_dataset pileup
    WHERE processed_dataset.name = :pileup_dataset
    """
    args = ArgSelector(kwds)
    
    if "pileup_dataset" not in kwds:
      return ()
    if "files_per_job" in kwds:
      return (PILEUP_INFO, args("pileup_dataset", "files_per_job"))
    return (PILEUP_INFO_DEFAULT_FILES_PER_JOB, args("pileup_dataset"))
      
  def createProcessingInfoQuery (self, kwds):
    PROCESSING_INFO_DEFAULTS = """INSERT INTO processing_info
    (input_dataset_id)
    SELECT (input.id)
    FROM processed_dataset input
    WHERE input.name = :input_dataset
    """
    
    PROCESSING_INFO = """INSERT INTO processing_info
    (input_dataset_id)
    SELECT (input.id, :split_size, s.id)
    FROM processed_dataset input
    INNER JOIN split_type s ON s.label = :split_type
    WHERE input.name = :input_dataset"""
    args = ArgSelector(kwds)
    if "input_dataset" not in kwds:
      return ()
    if "split_size" not in kwds:
      return (PROCESSING_INFO_DEFAULTS, args("input_dataset"))
    return (PROCESSING_INFO, args("input_dataset", "split_size", "split_type"))
    
  
  def createRequest (self, **kwds):
    args = ArgSelector(kwds)
    binds = BindsBuilder (kwds)
    binds.add ("event_type", "INJECTED")
    def __createDatasetIfNeeded(key):
      if key not in kwds:
        return
      if kwds[key] in ["input_dataset", "output_dataset", "pileup_dataset"]:
        kwds[key] = None
        return
      self.simpleInsert ("processed_dataset", name=kwds[key])
    __createDatasetIfNeeded("input_dataset")
    __createDatasetIfNeeded("output_dataset")
    __createDatasetIfNeeded("pileup_dataset")
    self.simpleInsert ("primary_dataset", name=kwds["primary_dataset"])

    def __updateRequest(item, creation_hash):
      UPDATE_REQUEST = """UPDATE request 
                          SET %s = last_insert_rowid() 
                          WHERE creation_hash = :creation_hash"""
      return (UPDATE_REQUEST % item, {"creation_hash" : creation_hash})
    
    creation_hash = ''.join([random.choice('abcdefghijklmnopqrstuvwxyz') for x in xrange(20)])
    CREATE_NEW_REQUEST = ("""INSERT INTO request (creation_hash) VALUES(:creation_hash);""", 
                          {"creation_hash": creation_hash})
    
    ADD_HISTORY_INFO = ("""INSERT INTO history_log
(event_type_id, time, request_id)
SELECT event_type.id, date('now'), last_insert_rowid()
FROM event_type
WHERE event_type.label = :event_type;
""", args ("event_type"))
    
    ADD_REQUESTOR_INFO = ("""INSERT INTO requestor_info 
(owner_id, physics_group_id)
SELECT :user_id, physics_group.id
FROM physics_group
WHERE physics_group.name = :physics_group;
""", args ("user_id", "physics_group"))

    ADD_CFG_FILE = ("""INSERT INTO cfg
(parameterset)
VALUES (:parameterset);""", args("parameterset"))

    ADD_PHYSICS_INFO = ("""INSERT INTO physics_info
(primary_dataset_id, selection_efficiency, cross_section, request_type_id, cfg_id) 
SELECT p.id, :selection_efficiency, :cross_section, rt.id, last_insert_rowid()
FROM request_type rt
INNER JOIN primary_dataset p ON p.name = :primary_dataset
WHERE rt.label = :request_type;
""", args ("primary_dataset","selection_efficiency", "cross_section", "request_type"))
    
    print args ("requested_events", "prodcommon_version", 
                "priority_hint", "preferred_consumer", "architecture", "cmssw_version", "request_category")
    
    ADD_PRODUCTION_INFO = ("""INSERT INTO production_info
(preferred_consumer_id, priority_hint, 
 requested_events, prodcommon_id, cmssw_id, request_category_id)
SELECT c.id, :priority_hint, :requested_events, pc.id, sw.id, rc.id
FROM consumer c
INNER JOIN prodcommon pc ON pc.label = :prodcommon_version
LEFT OUTER JOIN cmssw sw ON sw.version = :cmssw_version
LEFT OUTER JOIN request_category rc ON rc.label = :request_category
WHERE c.label = :preferred_consumer;
""", args ("requested_events", "prodcommon_version", 
            "priority_hint", "preferred_consumer", "architecture", 
            "cmssw_version", "request_category"))

    statements = []
    bindings = []
    
    tasks = [CREATE_NEW_REQUEST,
             ADD_HISTORY_INFO,
             __updateRequest("creation_log_entry_id", creation_hash),
             __updateRequest("current_log_entry_id", creation_hash),
             self.createPileupInfoQuery(kwds),
             __updateRequest("pileup_info_id", creation_hash),
             self.createProcessingInfoQuery(kwds),
             __updateRequest("processing_info_id", creation_hash),
             ADD_REQUESTOR_INFO,
             __updateRequest("requestor_info_id", creation_hash),
             ADD_CFG_FILE,
             ADD_PHYSICS_INFO,
             __updateRequest("physics_info_id", creation_hash),
             ADD_PRODUCTION_INFO,
             __updateRequest("production_info_id", creation_hash)]
    while tasks:
      task = tasks.pop (0)
      if not task:
        tasks.pop (0)
        continue
      query, binding = task
      statements.append(query)
      bindings.append(binding)
    self.editDataObject(statements, bindings)
    print [x for x in self.getDataTable("""SELECT * 
                                           FROM request 
                                           WHERE creation_hash = :creation_hash""", 
                                           {"creation_hash": creation_hash})]
    print [x for x in self.getDataTable("""SELECT * 
                                           FROM production_info 
                                           ORDER BY id DESC
                                           LIMIT 1;""", {})]
    print [x for x in self.getDataTable("""                                       SELECT c.id, :priority_hint, :requested_events, pc.id
                                           FROM consumer c
                                           INNER JOIN prodcommon pc ON pc.label = :prodcommon_version
                                           WHERE c.label = :preferred_consumer;
    
    """,                                        args ("requested_events", "prodcommon_version", 
                                                       "priority_hint", "preferred_consumer", "architecture"))]