from ProdRequestCommon.DataBase import DataBase
monitoringDB = DataBase ('monitoring.db')
monitoringDB.connect ()

from ProdRequestCommon.MonitoringCore.GlobalHistory import GlobalHistory
from ProdRequestCommon.MonitoringCore.GroupHistory import GroupHistory
from ProdRequestCommon.MonitoringCore.PhysicsGroupHistory import PhysicsGroupHistory
from ProdRequestCommon.MonitoringCore.UserHistory import UserHistory
from Models import EntryStatus
from sqlobject import *

from sqlobject import DateTimeCol
from Models import WorkflowEntry, Workflow
from datetime import timedelta


class MonitoringCore (object):
    def __init__ (self, context):
        self.context = context
        self.__logger = self.context.Logger ()
        GlobalHistory.createTable (ifNotExists=True)
        GroupHistory.createTable (ifNotExists=True)
        UserHistory.createTable (ifNotExists=True)
        PhysicsGroupHistory.createTable (ifNotExists=True)
        
    def harvestStats (self):
        harvestTime = DateTimeCol.now ()
        self.__getGlobalStats (harvestTime)
        self.__getGroupStats (harvestTime)
        self.__getUsersStats (harvestTime)
        self.__getPhysicsGroupStats (harvestTime)
        
    def __getGlobalStats (self, harvestTime):
        results = {}
        for key, condition in self.__getStatusConditions ().items ():
            results[key] = WorkflowEntry.select (AND (*condition)).count ()
        
        for key, condition in self.__getStatusConditions ().items ():
            event_sum = 0
            for entry in WorkflowEntry.select (AND (*condition)):
                event_sum += entry.requested_events
            results[key+"_events"] = event_sum
        
        GlobalHistory (timestamp=harvestTime, **results)

    def __getUsersStats (self, harvestTime):
        # get a list of all the available keys.
        workflows = Workflow.select ()
        uniqueIDs = {}
        for workflow in workflows:
            uniqueIDs[workflow.owner_id] = 1
        
        for userId in uniqueIDs.keys ():
            self.__logger.trace ("Get infos for user: %s" % userId)
            results = {}
            for key, condition in self.__getStatusConditions ().items ():
                results[key] = WorkflowEntry.select (AND (Workflow.q.owner_id == userId,
                                                          WorkflowEntry.q.workflowID == Workflow.q.id, *condition)).count ()
            for key, condition in self.__getStatusConditions ().items ():
                event_sum = 0
                for entry in WorkflowEntry.select (AND (Workflow.q.owner_id == userId,
                                                        WorkflowEntry.q.workflowID == Workflow.q.id, *condition)):
                    event_sum += entry.requested_events
                results[key+"_events"] = event_sum
            UserHistory (timestamp=harvestTime, user_id=userId, **results)
            

    def __getGroupStats (self, harvestTime):
        pass

    def __getPhysicsGroupStats (self, harvestTime):
        entries = WorkflowEntry.select ()
        uniqueIDs = {}
        for entry in entries:
            uniqueIDs[entry.physics_group] = 1
        
        for groupId in uniqueIDs.keys ():
            self.__logger.trace ("Getting info for the physics group %s" % groupId)
            results = {}
            for key, conditions in self.__getStatusConditions ().items ():
                results[key] = WorkflowEntry.select (AND (WorkflowEntry.q.physics_group == groupId, *conditions)).count ()
            
            for key, condition in self.__getStatusConditions ().items ():
                event_sum = 0
                for entry in WorkflowEntry.select (AND (WorkflowEntry.q.physics_group == groupId, *condition)):
                    event_sum += entry.requested_events
                results[key+"_events"] = event_sum
            PhysicsGroupHistory (harvest_time=harvestTime, physics_group=groupId, **results)

    def __getStatusConditions (self):
        conditions = {}
        conditions["pending"] = [WorkflowEntry.q.status == EntryStatus.PENDING]
        conditions["approved"] = [OR ([ WorkflowEntry.q.status == EntryStatus.APPROVED],
                                      [ WorkflowEntry.q.status == EntryStatus.APPROVED_PRODMANAGER])]
        conditions['running'] = [WorkflowEntry.q.status == EntryStatus.RUNNING]
        conditions['done'] = [WorkflowEntry.q.status == EntryStatus.DONE]
        conditions['requested'] = [OR([WorkflowEntry.q.status == EntryStatus.PENDING],
                                      [WorkflowEntry.q.status == EntryStatus.APPROVED],
                                      [WorkflowEntry.q.status == EntryStatus.RUNNING],
                                      [WorkflowEntry.q.status == EntryStatus.DONE])]
        return conditions
    
    def harvestIfNeeded (self):
        lastTime = GlobalHistory.get (GlobalHistory.select ().count ()).timestamp
        self.__logger.trace ("lastTime %s, now %s" % (lastTime , DateTimeCol.now ()))
        self.__logger.trace (DateTimeCol.now ())
        if DateTimeCol.now () - lastTime  > timedelta (seconds=3600*8):
            self.__logger.trace ("Update info")
            self.harvestStats ()
    
    def getGlobalProductionStatus (self):
        results = GlobalHistory.select ()
        if results.count ():
            self.harvestIfNeeded ()
            results = GlobalHistory.select ()
        else:
            self.harvestStats ()
        return GlobalHistory.get (results.count ())
    
    def getGlobalProductionHistory (self, timeSpan):
        self.harvestIfNeeded ()
        return GlobalHistory.select (GlobalHistory.q.timestamp >  DateTimeCol.now () - timeSpan)
    
    def getUserProductionStatus (self, userId):
        self.harvestIfNeeded ()
        results = UserHistory.select (UserHistory.q.user_id == userId)
        if not results.count ():
            self.__logger.trace ("No objects in history.")
            return None
        return results.reversed ()[0]
        
    def getPhysicsGroupProductionStatus (self, groupId):
        self.harvestIfNeeded ()
        results = PhysicsGroupHistory.select (PhysicsGroupHistory.q.physics_group == groupId)
        if not results.count ():
            return None
        return results.reversed ()[0]
