from Framework import Context
from Framework.Logger import Logger
from sqlobject import *

from xml.sax import make_parser, handler
from Models import Architecture, Project

class TagCollectorReleasesHandler(handler.ContentHandler):
    def __init__(self, context):
        self.context = context
        self.currentArchitecture = None
    
    def startElement(self, name, attributes):
        if name == "project":
            self.context.Logger ().debug ("Project found %s for architecture %s" % (attributes["label"], self.currentArchitecture.id))
            projects = Project.select (AND (Project.q.name == attributes["label"],
                                            Project.q.architecture_idID == self.currentArchitecture.id))
            if not projects.count ():
                self.context.Logger ().debug ("Project not found in DB, adding.")
                Project (name=attributes["label"], architecture_idID=self.currentArchitecture.id)
        elif name == "architecture":
            architectures = Architecture.select (Architecture.q.name == attributes["name"])
            if not architectures.count ():
                architecture = Architecture (name=attributes["name"])
                self.currentArchitecture = architecture
                return
            self.currentArchitecture = architectures[0]
    
    def endElement(self, name):
        if name == "architecture":
            self.currentArchitecture = None
    

class TagCollectorCore(object):
    def __init__ (self, tcUrl="https://cmstags.cern.ch/cgi-bin/CmsTC", context=Context ()):
        self.context = Context (context)
        self.context.addService (Logger ("TagCollectorCore"))
        self.tcUrl = tcUrl + "/ReleasesXML"
        self.context.Logger ().debug (self.tcUrl)
        
    def importTagCollectorInfo (self):
        parser = make_parser ()
        parser.setContentHandler(TagCollectorReleasesHandler (self.context))
        parser.parse (self.tcUrl)
