def declare_arg_converter (nativeName, converterType, converter):
    """ This class is a decorator for web methods which allows you to define
        a converter between (for example) a web usable id and an actual object.
        This is useful, for example, if you want to define a web method which
        refers to an object by id, but you want to work on the actual objects.
        
        Example:
        
        someDict = {
            1: "foo",
            2: "bar"
        }
        
        def getById (someId):
            return someDict (someId):
        
        @declare_arg_converter ('someArg', 'id', getById)
        def printString (someArg):
            print someArg
        
        printString (someArg_id=1)    
        ---
        This will print "foo" because the decorator will automatically convert
        someArg_id to string using the getById helper.        
    """
    converterName = "%(nativeName)s%(converterType)s" % locals ()
    def decorator (function):
        def wrapper (*__args, **__kwds):
            if converterName in __kwds:
                objectRef = __kwds[converterName]
                nativeObject = converter (objectRef)
                __kwds[nativeName] = nativeObject
            return function (*__args, **__kwds)
        return wrapper
    return decorator

