from datetime import datetime

import ProdCommon.MCPayloads.WorkflowTools as WorkflowTools
from ProdCommon.MCPayloads.WorkflowMaker import WorkflowMaker
from ProdCommon.CMSConfigTools.ConfigAPI.CMSSWConfig import CMSSWConfig
from os import getenv
import os

def log (sentence):
  print "<!--%s-->" % sentence
  
def createWorkflow (request_info):
    try:
        pwd = getenv ("PWD")
        oldCmsswSearchPath = getenv ("CMSSW_SEARCH_PATH")
        os.environ ["CMSSW_SEARCH_PATH"] = "%s:%s" % (pwd, oldCmsswSearchPath)
        # FIXME: ugly hack because FWCore does not support input from string.
        tmpCfgFilename = ("cfg"+entryId+ ".cfg").replace (" ","+")
        tmpCfgFile = file (tmpCfgFilename, "w")
        tmpCfgFile.write (request_info["cfg__parameterset"])
        tmpCfgFile.close ()
        cfgFile = tmpCfgFilename
        
        # for the time being we support only cfg file. We might have to support multiple types soon
        cfgType = "cfg"
        if cfgType == "cfg":
            from FWCore.ParameterSet.Config import include
            cmsCfg = include(cfgFile) 
        else:
            modRef = imp.find_module(os.path.basename(cfgFile).replace(".py", ""),  os.path.dirname(cfgFile))
            cmsCfg = modRef.process
        
        cfgWrapper = CMSSWConfig()
        cfgWrapper.originalCfg = file(cfgFile).read()
        cfgInt = cfgWrapper.loadConfiguration(cmsCfg)
        cfgInt.validateForProduction()
    except Exception, e:
        return "<error>Unable to create request: %s</error>" % e
    
    #  //
    # // Instantiate a WorkflowMaker
    #//
    # FIXME: label should be taken from the DB.
    maker = WorkflowMaker(request_info["id"], 
                          request_info["physics_info__primary_dataset"], 
                          request_info["production_info__request_type"])
    
    maker.setCMSSWVersion(request_info["cmssw__version"])
    maker.setPhysicsGroup(request_info["physics_group__name"]))
    maker.setConfiguration(cfgWrapper, Type = "instance")
    maker.setPSetHash(WorkflowTools.createPSetHash(cfgFile))
    
    # FIXME: category hardcoded to "mc". Might be nice to get it from DB as well. 
    category = "mc"
    maker.changeCategory(category)
    
    if request_info["physics_info__selection_efficiency"]:
        maker.addSelectionEfficiency(request_info["physics_info__selection_efficiency"])
    
    # 
    # Pileup sample?
    # 
    badNames = ("", None, "input_dataset", "pileup_dataset", "None", "undefined")
    if not request_info["pileup_dataset__name" in badNames:
        maker.addPileupDataset(request_info["pileup_dataset__name"], 
                               request_info["pileup_info__files_per_job"])
    
    
    if not request_info["input_dataset__name"] in badNames:
        log ("Input dataset: %s" % request_info["input_dataset__name"])
        maker.addInputDataset (request_info["input_dataset__name"])
        maker.inputDataset['SplitType'] = request_info["processing_info__split_type"]
        maker.inputDataset['SplitSize'] = request_info["processing_info__split_size"]


    spec = maker.makeWorkflow()
    
    log ("Created: %s-Workflow.xml" % maker.workflowName)
    log ("From: %s " % cfgFile)
    log ("Output Datasets:")

    [ log (
         "/%s/%s/%s\n" % (
           x['PrimaryDataset'],
           x['ProcessedDataset'],
           x['DataTier'])) for x in spec.outputDatasets()]
    result = spec.saveString()
    return result        

if __name__ == "__main__":
    request_info = sys.stdin.read()
    print createWorkflow(request_info)
