# Standard stuff to include for ProdCommonPlugin
# 
from datetime import datetime

import ProdCommon.MCPayloads.WorkflowTools as WorkflowTools
from ProdCommon.MCPayloads.WorkflowMaker import WorkflowMaker
from ProdCommon.CMSConfigTools.ConfigAPI.CMSSWConfig import CMSSWConfig
from ProdCommon.MCPayloads.CmsGenWorkflowMaker import CmsGenWorkflowMaker

from os import getenv, popen
import os

def createWorkflow (request_info):
    try:
        pwd = getenv ("PWD")
        oldCmsswSearchPath = getenv ("CMSSW_SEARCH_PATH")
        os.environ ["CMSSW_SEARCH_PATH"] = "%s:%s" % (pwd, oldCmsswSearchPath)
        # FIXME: ugly hack because FWCore does not support input from string.
        tmpCfgFilename = ("cfg%s.cfg" % request_info["request__id"]).replace (" ","+")
        tmpCfgFile = file(tmpCfgFilename, "w")
        tmpCfgFile.write(request_info["cfg__parameterset"])
        tmpCfgFile.close()
        cfgFile = tmpCfgFilename
        
        # for the time being we support only cfg file. We might have to support multiple types soon
        cfgType = "cfg"
        if cfgType == "cfg":
            from FWCore.ParameterSet.Config import include
            cmsCfg = include(cfgFile) 
        else:
            modRef = imp.find_module( os.path.basename(cfgFile).replace(".py", ""),  os.path.dirname(cfgFile))
            cmsCfg = modRef.process
        
        cfgWrapper = CMSSWConfig()
        cfgWrapper.originalCfg = file(cfgFile).read()
        cfgInt = cfgWrapper.loadConfiguration(cmsCfg)
        cfgInt.validateForProduction()
    except Exception, e:
        return "<error>Unable to create request: %s</error>" % e
    
    # 
    # Instantiate a WorkflowMaker
    # 
    primary_dataset_name = "%s-%s" % (request_info["physics_info__primary_dataset"], request_info["generator_info__generator"])
    maker = CmsGenWorkflowMaker (request_info["request__id"],
                                 primary_dataset_name,
                                 request_info["production_info__request_category"])
    
    maker.setCMSSWVersion(request_info["cmssw__version"])
    maker.setPhysicsGroup(IdToPhysicsGroup (request_info["physics_group__name"]))
    maker.setConfiguration(cfgWrapper, Type = "instance")
    maker.setCmsGenConfiguration(request_info["generator_info__cfg"])
    maker.setCmsGenParameters(generator=request_info["generator_info__generator"])

    #psethash=popen ("export old=$PWD; export SCRAM_ARCH=slc4_ia32_gcc345; cd /afs/cern.ch/cms/sw/slc4_ia32_gcc345/cms/cmssw/CMSSW_1_4_6; eval `scramv1 run -sh`; cd $old; cat %(cfgFile)s | EdmConfigHash" % locals()).read()
    psethash=WorkflowTools.createPSetHash(cfgFile)
    maker.setPSetHash(psethash)
    
    maker.changeCategory(request_info["physics_info__request_category"])
    
    if request_info["physics_info__selection_efficiency"]:
        maker.addSelectionEfficiency(request_info["physics_info__selection_efficiency"])
        maker.addCmsGenSelectionEfficiency(request_info["physics_info__selection_efficiency"])
    
    # 
    # Pileup sample?
    # 
    badNames = (None, "input_dataset", "", "pileup_dataset", "None", "undefined")
    if not request_info["pileup_dataset__name"] in badNames:
        maker.addPileupDataset (request_info["pileup_dataset__name"], 
                                request_info["pileup_info__files_per_job")

    if not request_info["input_dataset__name"] in badNames:
        log ("Input dataset: %s" % request_info["input_dataset__name"])
        maker.addInputDataset(request_info["input_dataset__name"])
        maker.inputDataset['SplitType'] = request_info["processing_info__split_type"]
        maker.inputDataset['SplitSize'] = request_info["processing_info__split_size"]
    
    spec = maker.makeWorkflow()
    
    log ("Created: %s-Workflow.xml" % maker.workflowName)
    log ("From: %s " % cfgFile)
    log ("Output Datasets:")

    [ log (
         "/%s/%s/%s\n" % (
           x['PrimaryDataset'],
           x['ProcessedDataset'],
           x['DataTier'])) for x in spec.outputDatasets()]
    result = spec.saveString ()
    return result        

if __name__ == "__main__":
    request_info = eval(sys.stdin.read(),{},{})
    print createWorkflow(request_info)
