import sys
from optparse import OptionParser
from os.path import dirname,abspath
from tempfile import mkstemp
from os import popen, fdopen, unlink
from commands import getstatusoutput
import re

def getProdCommonVersion (prodcommon_plugin_label, prodcommon_root):
  prodcommonMatcher = re.compile(prodcommon_plugin_label.split ("_X")[0] + ".*")
  error, output = getstatusoutput("ls %(prodcommon_root)s" % locals())
  print output
  candidates = [directory
                for directory in output.split("\n")
                if prodcommonMatcher.match(directory)]
  return candidates[0]

def callWorkflowCreator (request_info, opts, args):
  # This helper function does the following:
  # * Gets the path where to find the various workflowCreatorXXX scripts.
  # * Creates a script that does the following:
  #   * Sets up the environment, as requested by prodcommon
  #   * Sets up the environment, as requested by cmssw
  #   * Execute the workflowCreatorXXX script for the given prodcommon
  # * Executes the generated script.
  scriptsPath = dirname(abspath(__file__))
  cmsSwRoot = opts.cmsSwRoot
  prodcommonRoot = opts.prodcommonRoot
  architecture = opts.architecture
  prodcommonPlugin = request_info["prodcommon__label"]
  prodcommonVersion = getProdCommonVersion(request_info["prodcommon__label"], prodcommonRoot) 
  cmsswVersion = request_info["cmssw__version"]
  cmsswArchitecture = request_info["architecture__name"]
  dbsApiRoot = opts.dbsApiRoot
  workflowScript = """#!/bin/sh
source %(prodcommonRoot)s/%(prodcommonVersion)s/etc/profile.d/init.sh
source %(dbsApiRoot)s/etc/profile.d/init.sh
export OLD_PWD=`pwd`
cd %(cmsSwRoot)s/%(cmsswArchitecture)s/cms/cmssw/%(cmsswVersion)s
eval `scram run -sh`
mkdir -p $OLD_PWD/cfgs
cd $OLD_PWD/cfgs
cat << \\EOF | python %(scriptsPath)s/workflowCreation%(prodcommonPlugin)s.py
%(request_info)s
EOF
""" % locals ()
  fd, fname = mkstemp ()
  f=fdopen(fd, 'w')
  f.write(workflowScript % locals())
  f.close()
  try:
    results = popen("sh %(fname)s" % locals()).read()
    print results
  except Exception, e:
    print e
  unlink(fname)
  
if __name__ == "__main__":
  parser = OptionParser()
  parser.add_option("--cmssw-root", dest="cmsSwRoot")
  parser.add_option("--prodcommon-root", dest="prodcommonRoot")
  parser.add_option("--prodcommon-architecture", dest="architecture")
  parser.add_option("--prodcommon-version", dest="prodcommonVersion")
  parser.add_option("--dbs-api-root", dest="dbsApiRoot")
  parser.add_option("--debug", dest="debug", action="store_true", default=False)
  opts, args = parser.parse_args ()
  request_info = eval (sys.stdin.read (), {}, {})
  if opts.debug:
    print request_info
    sys.exit(0)
  callWorkflowCreator(request_info, opts, args)
  sys.exit(0)
  