#!/usr/bin/env python
from os import getenv
from sys import argv
from xml.sax import parse
from xml.sax.handler import ContentHandler
from os.path import abspath, exists
from optparse import OptionParser

#from Utilities.InjectionUtilities import _call, createPSetHash, createPythonConfig

###
###
###
import popen2
import sys

def createPSetHash(cfgFile):
    hashFile = "%s.hash" % cfgFile
    pop = popen2.Popen4("EdmConfigHash < %s > %s " % (cfgFile, hashFile))
    pop.wait()
    exitStatus = pop.poll()
    if exitStatus:
        msg = "Error creating PSet Hash file:\n"
        msg += pop.fromchild.read()
        raise RuntimeError, msg

    content = file(hashFile).read()
    return content.strip()

def createPythonConfig(cfgFile):
    pycfgFile = cfgFile.replace(".cfg", ".pycfg")
    pop = popen2.Popen4("EdmConfigToPython < %s > %s " % (cfgFile, pycfgFile))
    pop.wait()
    exitStatus = pop.poll()
    if exitStatus:
        msg = "Error creating Python cfg file:\n"
        msg += pop.fromchild.read()
        raise RuntimeError, msg

    pop = popen2.Popen4("%s %s" % (sys.executable, pycfgFile))
    pop.wait()
    exitStatus = pop.poll()
    if exitStatus:
        msg = "Error importing Python cfg file:\n"
        msg += pop.fromchild.read()
        raise RuntimeError, msg

    return pycfgFile

from cStringIO import StringIO
import urllib2
import gzip 

def _encode(args):
    boundary = '----------=_PRODREQUEST_CGI_BOUNDARY_=-----------'
    (body, crlf) = ('', '\r\n')
    for key, value in args.items():
        body += '--' + boundary + crlf
        body += ('Content-disposition: form-data; name="%s"' % key) + crlf
        body += crlf + value + crlf
    body += '--' + boundary + '--' + crlf + crlf
    return ('multipart/form-data; boundary=' + boundary, body)

def _marshall(args, request):
    (type, body) = _encode(args)
    request.add_header ('Content-type', type)
    request.add_header ('Content-length', str(len(body)))
    request.add_data (body)

def _call (url, args):
    try:
        #request = urllib2.Request (self._cgiUrl, urllib.urlencode(args))
        request = urllib2.Request (url)
        request.add_header ('Accept-encoding', 'gzip')
        _marshall (args, request)
        result = urllib2.build_opener ().open (request)
        data = result.read()
        if result.headers.get ('Content-encoding', '') == 'gzip':
            data = gzip.GzipFile (fileobj=StringIO(data)).read ()
        return data
    except:
        pass

###
###
###


class RelValContentHandler (ContentHandler):
    def __init__ (self, version, relval, directory, hostAndPort):
        ContentHandler.__init__ (self)
        self.events = None
        self.testStarted = False
        self.workflowName = None
        self.cfgFile = None
        self.version = version
        self.relval = relval
        self.directory = directory
        self.hostAndPort = hostAndPort

    def startElement (self, name, attributes):
        if name == "Test":
            self.events = attributes.getValueByQName ("Events")
            self.workflowName = attributes.getValueByQName ("Name")
            self.testStarted = True
    
    def characters (self, data):
        if self.testStarted:
            if data.strip (" \t\n") != "":
                self.cfgFile = data.strip ("\n\t ")

    def endElement (self, name):
        if name == "Test":
            print self.events
            print self.workflowName
            print self.cfgFile
            self.inject ()
            self.testStarted = False
        pass

    def inject (self):
        prodrequestUrl="http://%s/ProdRequest/createPayloadWithEntry" % self.hostAndPort
        cfgfile = "%s/%s" % (self.directory, self.cfgFile) 
        
        if not exists (cfgfile):
            print cfgfile + " not found."
            pass
        
        cfg = file (cfgfile).read ()
        pycfgfile = createPythonConfig(cfgfile)
        pycfg = file (pycfgfile).read ()
        psetHash = createPSetHash (cfgfile)

        cfgname = "%s - %s" % (self.relval, self.version)
        _call (prodrequestUrl,
            {"name": self.workflowName,
            "parameterset": cfg,
            "parsed_parameterset": pycfg,
            "pset_hash": psetHash,
            "primary_dataset": self.workflowName,
            "output_dataset": "output_dataset",
            "workflowName": cfgname,
            "version": self.version})
        
        
def injectRequest (confFilename, hostAndPort):
    confFilename = abspath (confFilename)
    print confFilename
    directory = confFilename.rsplit ("/", 1)[0]
    relval = confFilename.rsplit ("/", 1)[1].rsplit (".", 1)[0]
    cmsswVersion = getenv ("CMSSW_VERSION")
    if cmsswVersion:
        print "CMSSW version is " + cmsswVersion
    handler = RelValContentHandler (version=cmsswVersion, relval=relval, directory=directory, hostAndPort=hostAndPort)
    parser = parse (confFilename, handler)

if __name__ == "__main__":
    optionParser = OptionParser ()
    optionParser.add_option ("-u", "--url", dest="url", 
                             metavar="URL", default="localhost:8030",
                             help="ProdRequest url")
    optlist, args = optionParser.parse_args ()
    for i in args:
        injectRequest (i, optlist.url)
