#!/bin/sh
rm -f prodrequest_new.db
rm -f prodrequest_gui.db
sqlite3 prodrequest_gui.db < Models/prodrequest_gui.sql
sqlite3 prodrequest_new.db < Models/prodrequest.sql

sh ./migrate-sitedb.sh

# TODO: migrate the location_workflow_entry table;
# Use a mask rather than actual id.
sqlite3 << \EOF
ATTACH "prodrequest_old.db" AS pr_old;
ATTACH "prodrequest_new.db" AS pr_new;

INSERT INTO pr_new.production_location
(request_id, location_id)
SELECT we.id, l.id 
FROM pr_old.workflow_entry we
INNER JOIN pr_old.location_workflow_entry lwe ON lwe.workflow_entry_id = we.id
INNER JOIN pr_new.location l ON l.id = (1 << lwe.location_id);
EOF

# Make sure all the datasets are now in the correct tables.
sqlite3 << \EOF
ATTACH "prodrequest_old.db" AS pr_old;
ATTACH "prodrequest_new.db" AS pr_new;

SELECT "---";
SELECT "# Make sure all the datasets are now in the correct tables.";

INSERT INTO pr_new.primary_dataset (name) SELECT primary_dataset FROM pr_old.workflow_entry;
INSERT INTO pr_new.processed_dataset (name) SELECT input_dataset FROM pr_old.workflow_entry;
INSERT INTO pr_new.processed_dataset (name) SELECT output_dataset FROM pr_old.workflow_entry;
INSERT INTO pr_new.processed_dataset (name) SELECT pileup_dataset FROM pr_old.workflow_entry;
SELECT * from pr_new.processed_dataset limit 5;
SELECT * from pr_new.primary_dataset limit 5;
EOF

# Migrate the various projects to the new cmssw table.
sqlite3 << \EOF
ATTACH "prodrequest_old.db" AS pr_old;
ATTACH "prodrequest_new.db" AS pr_new;
SELECT "---";
SELECT "# Migrate the various projects to the new cmssw table.";

INSERT INTO pr_new.cmssw
(id, version, architecture_id)
SELECT old_project.id, old_project.name, old_project.architecture_id_id
FROM pr_old.project old_project;
SELECT * FROM cmssw LIMIT 5;
EOF

# Migrate the processing_info
sqlite3 << \EOF
ATTACH "prodrequest_old.db" AS pr_old;
ATTACH "prodrequest_new.db" AS pr_new;

SELECT "---";
SELECT "# Migrate the processing_info";

INSERT INTO pr_new.processing_info
(id, input_dataset_id)
SELECT old_we.id, new_pd.id
FROM pr_old.workflow_entry old_we
INNER JOIN pr_new.processed_dataset new_pd ON new_pd.name = old_we.input_dataset 
WHERE old_we.input_dataset != "input_dataset" AND old_we.input_dataset;
SELECT * FROM pr_new.processing_info LIMIT 10;
EOF

# Migrate the pileup_info
sqlite3 << \EOF
ATTACH "prodrequest_old.db" AS pr_old;
ATTACH "prodrequest_new.db" AS pr_new;

SELECT "---";
SELECT "# Migrate the pileup_info";

INSERT INTO pr_new.pileup_info
(id, pileup_dataset_id)
SELECT old_we.id, new_pd.id
FROM pr_old.workflow_entry old_we
INNER JOIN pr_new.processed_dataset new_pd ON new_pd.name = old_we.pileup_dataset
WHERE old_we.pileup_dataset != "pileup_dataset" AND old_we.pileup_dataset;
SELECT * FROM pr_new.pileup_info LIMIT 10;
EOF

# Migrate the production info in a separate table.
sqlite3 << \EOF
ATTACH "prodrequest_old.db" AS pr_old;
ATTACH "prodrequest_new.db" AS pr_new;

SELECT "---";
SELECT "# Migrate the production info in a separate table.";

INSERT INTO pr_new.production_info
(id, priority_hint, requested_events, prodcommon_id, 
 preferred_consumer_id, cmssw_id)
SELECT old_r.id, old_r.priority_hint, old_r.requested_events, new_pc.id,
       old_r.preferred_consumer, old_r.project_id
FROM pr_old.workflow_entry old_r
INNER JOIN pr_new.prodcommon new_pc ON new_pc.label = old_r.prodcommon_version;
SELECT * from pr_new.production_info LIMIT 10;
EOF

# Migrate the creation date to use the new history_log.
sqlite3 << \EOF
ATTACH "prodrequest_old.db" AS pr_old;
ATTACH "prodrequest_new.db" AS pr_new;

SELECT "---";
SELECT "# Migrate the creation date to use the new history_log.";

INSERT INTO pr_new.history_log
(event_type_id, request_id, time)
SELECT new_et.id, old_we.id, old_w.creation_time
FROM pr_old.workflow_entry old_we
CROSS JOIN pr_new.event_type new_et
INNER JOIN pr_old.workflow old_w ON old_w.id = old_we.workflow_id
WHERE new_et.label = "INJECTED";
SELECT * FROM pr_new.history_log LIMIT 10;
EOF

# Migrate the stautus to use the new history_log.
sqlite3 << \EOF
ATTACH "prodrequest_old.db" AS pr_old;
ATTACH "prodrequest_new.db" AS pr_new;

SELECT "---";
SELECT "# Migrate the stautus to use the new history_log.";

INSERT INTO pr_new.history_log
(event_type_id, request_id, time)
SELECT old_we.status, old_we.id, datetime('now')
FROM pr_old.workflow_entry old_we;
SELECT * FROM pr_new.history_log log WHERE log.event_type_id != 0 LIMIT 10;
EOF

# Migrate the physics info in a separate table.
sqlite3 << \EOF
ATTACH "prodrequest_old.db" AS pr_old;
ATTACH "prodrequest_new.db" AS pr_new;

SELECT "---";
SELECT "# Migrate the physics info in a separate table.";

INSERT INTO pr_new.cfg
(parameterset)
SELECT parameterset from payload;

INSERT INTO pr_new.physics_info
(id, primary_dataset_id, cross_section, selection_efficiency, cfg_id)
SELECT old_r.id, new_pd.id, old_r.cross_section, old_r.selection_efficiency, new_cfg.id
FROM pr_old.workflow_entry old_r
INNER JOIN pr_new.primary_dataset new_pd ON new_pd.name = old_r.primary_dataset
LEFT OUTER JOIN pr_old.payload old_p ON old_p.id = old_r.payload_id
LEFT OUTER JOIN pr_new.cfg new_cfg ON new_cfg.parameterset = old_p.parameterset;
SELECT * FROM physics_info LIMIT 10;
EOF

# Migrate the owner_id and all the rest to use a requestor table.
sqlite3 << \EOF
ATTACH "prodrequest_old.db" AS pr_old;
ATTACH "prodrequest_new.db" AS pr_new;

INSERT INTO pr_new.requestor_info
(id, physics_group_id, owner_id)
SELECT old_r.id, old_r.physics_group, um.new_id
FROM workflow_entry old_r
LEFT OUTER JOIN pr_new.user_mapping um ON um.old_id = old_r.owner_id;

SELECT "---";
SELECT * FROM pr_new.requestor_info LIMIT 10;
SELECT "---";
EOF

# Migrate spec_info to request_spec
sqlite3 << \EOF
ATTACH "prodrequest_old.db" AS pr_old;
ATTACH "prodrequest_new.db" AS pr_new;
INSERT INTO pr_new.request_spec
(id, text)
SELECT old_we.id, old_si.text
FROM pr_old.workflow_entry old_we
INNER JOIN pr_old.spec_info old_si ON old_si.id = old_we.spec_info_id;
EOF

# Migrate the generator_info_id
sqlite3 << \EOF
ATTACH "prodrequest_old.db" AS pr_old;
ATTACH "prodrequest_new.db" AS pr_new;

INSERT INTO pr_new.generator_info
(id, executable, generator, cfg)
SELECT old_we.id, old_gi.executable, old_gi.generator, old_gi.cfg
FROM pr_old.workflow_entry old_we
INNER JOIN pr_old.generator_info old_gi ON old_gi.id = old_we.generator_info_id;
SELECT * FROM pr_new.generator_info LIMIT 10;
EOF

# Migrate the workflow_entry to be a request
sqlite3 << \EOF
ATTACH "prodrequest_old.db" AS pr_old;
ATTACH "prodrequest_new.db" AS pr_new;

INSERT INTO pr_new.request
(id, requestor_info_id, physics_info_id, production_info_id,
 creation_log_entry_id, current_log_entry_id, generator_info_id, request_spec_id, processing_info_id, pileup_info_id)
SELECT old_we.id, old_we.id, old_we.id, old_we.id, 
       new_ch.id, new_cs.id, gi.id, rs.id, processing.id, pileup.id
FROM pr_old.workflow_entry old_we
INNER JOIN pr_new.history_log new_ch ON new_ch.request_id = old_we.id AND new_ch.event_type_id = 0
INNER JOIN pr_new.history_log new_cs ON new_cs.request_id = old_we.id AND new_cs.event_type_id = old_we.status AND new_cs.event_type_id != 0
LEFT OUTER JOIN pr_new.generator_info gi ON gi.id = old_we.id
LEFT OUTER JOIN pr_new.request_spec rs ON rs.id = old_we.id
LEFT OUTER JOIN pr_new.processing_info processing ON processing.id = old_we.id
LEFT OUTER JOIN pr_new.pileup_info pileup ON pileup.id = old_we.id;
SELECT * FROM pr_new.request LIMIT 10;
EOF

# Migrate the approved_request table
sqlite3 << \EOF
ATTACH "prodrequest_old.db" AS pr_old;
ATTACH "prodrequest_new.db" AS pr_new;

INSERT INTO pr_new.approved_request
(id, request_id, consumer_id)
SELECT old_ap.id, old_ap.entry_id, old_ap.consumer 
FROM pr_old.approved_request old_ap;
SELECT * FROM pr_new.approved_request LIMIT 10;
EOF


# TODO: change the spec_info table to use blobs.
